//***************************************************************************************
// 
// This application demonstrates how to use the Microvision ActiveX Control to create a
// scanner interface.
// 
// Copyright  2007 Microvision, Inc.
// All Rights Reserved.
//
//---------------------------------------------------------------------------------------
//
// Date			Author			Description
// ----------	--------------	--------------------------------------------------------
// 2007-11-18	T Cataldo		Added this comment block.
//
//***************************************************************************************


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;


namespace BcsActiveX.CS_NET
{
	public partial class BcsActiveX : Form
	{
		public BcsActiveX()
		{
			InitializeComponent();
		}

		///--------------------------------------------------------------------------------
		/// <summary>
		/// Use this function when you want to search for Bluetooth scanners. The Bluetooth
		/// scanners found will be displayed in the control. In order for a scanner to be
		/// found it must not be paired to another computer and it must be in Discoverable
		/// mode. Place the scanner in Discoverable mode by pressing the scan button. The
		/// scanner should repeatedly double-blink the blue LED when it is Discoverable.
		/// </summary>
		///--------------------------------------------------------------------------------
		private void ButtonAdd_Click(object sender, EventArgs e)
		{
			try
			{
				BcsConn1.AddBluetoothDevice();
			}
			catch (Exception err)
			{
				MessageBox.Show (err.Message, "Bcs Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}		// end of ButtonAdd_Click


		///--------------------------------------------------------------------------------
		/// <summary>
		/// Configure the scanner by setting it to its factory default state and then
		/// setting the configuration parameters required by this application. In a "real"
		/// application you could also set the bar code parameters, symbology IDs, etc.
		/// </summary>
		///--------------------------------------------------------------------------------
		private void ButtonConfigure_Click(object sender, EventArgs e)
		{
			try
			{
				BcsConn1.Command("RestoreFactorySettings", null);	// Set scanner to a known configuration.
				BcsConn1.SetProp("AutoDownload", true);				// Send ATTN packet upon connection.
				BcsConn1.SetProp("BcDataAck", true);				// M2M with data acknowledgements (required).
				BcsConn1.SetProp("Time", System.DateTime.Now);		// Set scanner date/time to current value.
			}
			catch (Exception err)									// An error occurred when trying to configure
			{														//   the scanner.
				MessageBox.Show (err.Message, "Configure", MessageBoxButtons.OK, MessageBoxIcon.Information);
				return;
			}
			MessageBox.Show("Scanner Configured!", "Configure", MessageBoxButtons.OK, MessageBoxIcon.Information);
		}		// end of ButtonConfigure_Click


		///--------------------------------------------------------------------------------
		/// <summary>
		/// This function display the properties of the scanner connection. The properties
		/// are shown in a standard Windows property dialog. This can be very helpful when
		/// trying to troubleshoot connection problems. This function works even if you do
		/// not have an active scanner connection.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		///--------------------------------------------------------------------------------
		private void ButtonProperties_Click(object sender, EventArgs e)
		{
			try
			{
				BcsConn1.ShowDeviceProperties();
			}
			catch (Exception err)
			{
				MessageBox.Show(err.Message, "Bcs Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}		// end of ButtonProperties_Click
		
		
		///--------------------------------------------------------------------------------
		/// <summary>
		/// This function removes a selected connection line from the scanner control list.
		/// This is most often used to remove a Bluetooth scanner that is no longer being
		/// used by this host.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		///--------------------------------------------------------------------------------
		private void ButtonRemove_Click(object sender, EventArgs e)
		{
			try
			{
				BcsConn1.RemoveBluetoothDevice();
			}
			catch (Exception err)
			{
				MessageBox.Show(err.Message, "Bcs Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}		// end of ButtonRemove_Click


		///--------------------------------------------------------------------------------
		/// <summary>
		/// This function opens and closes a scanner connection and enables/disables the
		/// dialog controls as appropriate.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		///--------------------------------------------------------------------------------
		private void ButtonStart_Click(object sender, EventArgs e)
		{
			try
			{
				if (ButtonStart.Text == "Activate")
				{
					// Open a connection to the scanner and 0 auto-selects the baud rate.
					BcsConn1.Open(0);
					// Disable the scanner control to prevent click and other events.
					BcsConn1.Enabled = false;
					// Update the dialog controls.
					ButtonStart.Text = "Deactivate";
					ButtonConfigure.Enabled = true;
					CommandGetProp.Enabled  = true;
					CommandSetProp.Enabled  = true;
					
					// If the scanner is connected when the application starts then this command
					// causes the scanner to send its stored bar codes.
					BcsConn1.GetBarCodes ();
				}
				else
				{
					BcsConn1.Close();						// Close the open scanner connection.
					BcsConn1.Enabled = true;				// Enable the scanner list box control.
					ButtonStart.Text = "Activate";			// Update the dialog controls.
					ButtonConfigure.Enabled = false;
					CommandGetProp.Enabled  = false;
					CommandSetProp.Enabled  = false;
				}
			}		// end of try
			catch (Exception err)
			{
				MessageBox.Show (err.Message, "Bcs Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}		// end of ButtonStart_Click


		///--------------------------------------------------------------------------------
		/// <summary>
		/// This is the callback function that is called when the host detects data from
		/// the scanner. You never call this function directly -- the system calls it in
		/// response to a DataReceived event. Use this function to process the incoming
		/// bar code data. For this example we write the data to the screen but you could
		/// write the data to a file, add it to a database, etc. When adding this callback
		/// to an existing application select the BcsConn1 object from the "Object" drop-
		/// list above the IDE code window (list on the left). After selecting the BcsConn1
		/// object from the left then select the "BarCode" event in the right drop-list.
		/// The IDE will then create a function shell for your callback.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		///--------------------------------------------------------------------------------
		private void BcsConn1_BarCode(object sender, AxBcsCtlLib._IBcsConnEvents_BarCodeEvent eventArgs)
		{
			// For this example we display the bar code and the bar code details.
			OutputWindow.SelectedText = 
				eventArgs.barCode +
				"\t(si: " + eventArgs.symbology +
				", ts: "  + eventArgs.timeStamp +
				", tt: "  + eventArgs.timeType  + ")\r\n";
		}		// end of BcsConn1_BarCode


		///--------------------------------------------------------------------------------
		/// <summary>
		/// Use this function to read the specified property value from the scanner. In
		/// the dialog box select the property from the list and press the "Get" button.
		/// The program then displays the value for that scanner property. This function
		/// is very useful for verifying the scanner configuration and for verifying the
		/// scanner connection.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		///--------------------------------------------------------------------------------
		private void CommandGetProp_Click(object sender, EventArgs e)
		{
			try
			{
				// Get the value of the property displayed in the combo box on the main dialog.
				edPropValue.Text = BcsConn1.GetProp(cbPropName.Text).ToString();
			}
			catch (Exception err)
			{
				edPropValue.Text = "";
				MessageBox.Show(err.Message, "Bcs Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}		// end of CommandGetProp_Click


		///--------------------------------------------------------------------------------
		/// <summary>
		/// Set the value of the specified property. Select the property from the combo box
		/// on the bottom left in the main dialog. Enter the property value in the edit
		/// field on the bottom right of the dialog. When you press the "Set" button then
		/// the "SetProp" command is sent to the scanner. Note that this application does
		/// not validate the values that you specify. For example, if you set BcDataAck
		/// then valid values are "true" and "false". If you enter an inappropriate value
		/// then the scanner will reject the command.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		///--------------------------------------------------------------------------------
		private void CommandSetProp_Click(object sender, EventArgs e)
		{
			try
			{
				// Send the command specified in the combo box and the value from the edit field.
				BcsConn1.SetProp(cbPropName.Text, edPropValue.Text);
			}
			catch (Exception err)
			{
				MessageBox.Show(err.Message, "Bcs Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}		// end of CommandSetProp_Click


		///--------------------------------------------------------------------------------

	}		// end of class
}		// end of namespace