//***************************************************************************************
// 
// .NET API for the Microvision BarCode Scanner 
// 
// Copyright  2007 Microvision, Inc.
// All Rights Reserved.
//
//***************************************************************************************
using System;
using System.Runtime.InteropServices;
using System.Runtime.InteropServices.ComTypes;

namespace Microvision.BarCodeScanner
{
    /// <summary>
    /// 
    /// </summary>
    /// <param name="pvParam"></param>
    /// <param name="eventType"></param>
    /// <param name="pEvent"></param>
    /// <returns></returns>
    public delegate uint EventCallback(
        IntPtr pvParam,
        Scanner.EventType eventType,
        IntPtr pEvent);

    /// <summary>
	/// The Scanner class provides a .NET interface to the C-language Bar Code Scanner SDK DLL functions. The SDK
	/// DLL uses standard C objects such as single-byte, null-terminated strings. This wrapper returns
	/// proper .NET objects and handles all of the C to .NET conversions.
	/// </summary>
	public class Scanner : Exception
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="pvParam"></param>
		/// <param name="symbology"></param>
		/// <param name="timeStamp"></param>
		/// <param name="timeType"></param>
		/// <param name="barCode"></param>
		/// <param name="cchBarCode"></param>
		/// <param name="downloadInfo"></param>
		/// <returns></returns>
		public delegate uint BarCodeCallback(
			IntPtr	pvParam,	        			    // Parameter passed into BcsGetBarCodesA as the pvParam
			string	symbology,					        // Symbol Identifier (This value is dependent on the setting of the SymIdType property)
			string	timeStamp,					        // Timestamp (This pointer value is only valid if Timestamp = True, otherwize the the pointer value is NULL)
			string	timeType,					        // Time Type (This pointer value is only valid if Timestamp = True, otherwize the the pointer value is NULL)
			                                            // Possible Values: "ok" = Actual Time, or e1, e2, ..., e<N> = Estimated time after N battery events)
			string	barCode,					        // Bar code data
			int		cchBarCode,					        // Length of the bar code data in characters
			// BCS_DOWNLOAD_INFO *pDownloadInfo);	    // Supplemental information in download packet
            int downloadInfo);


		//-------------------------------------------------------------------------------------------------

		#region Constants
		//-----Version Information-------------------------------------------------------------------------

		private const int		BCS_VER_MAJOR  = 0;			// BCS_VER_MAJOR will be set to 1, and BCS_VER_MINOR set to 0 for the first release candidate.
		private const int		BCS_VER_MINOR  = 7;			// BCS_VER_MINOR will be incremented for each change to the interface before release.
		private const int		BCS_VER_BUILD  = 7;			// BCS_VER_BUILD will be incremented with each internal release and reset to 0 for each version number change.

		// This version information is stored as a resource in Bcs.dll which can be displayed
		// using Windows Explorer or programmatically through GetFileVersionInfo, VerQueryValue, etc.

		private const string	BCS_COMPANY_NAME		= "Microvision, Inc.";
		private const string	BCS_FILE_DESCRIPTION	= "Scanner API";
		private const string	BCS_FILE_VERSION		= "0.7.0.7";
		private const string	BCS_INTERNAL_NAME		= "Bcs";
		private const string	BCS_LEGAL_COPYRIGHT	    = "Copyright  2007 Microvision, Inc.";
		private const string	BCS_ORIGINAL_FILENAME	= "Bcs.dll";
		private const string	BCS_PRODUCT_NAME		= "Scanner SDK";
		private const string	BCS_PRODUCT_VERSION	    = BCS_FILE_VERSION;
		
        private const string    DLL_FILENAME            = "Bcs.dll";

		#endregion

		#region structures

        /// <summary><para><c>SystemTime</c> record.</para></summary>
        [StructLayout(LayoutKind.Sequential)]
        public struct SystemTime // RECORD
        {
            /// <summary><para><c>wYear</c> field of <c>SystemTime</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>wYear</c> field was the following:  <c>unsigned short wYear</c>;</para></remarks>
            // IDL: unsigned short wYear;
            public ushort wYear;

            /// <summary><para><c>wMonth</c> field of <c>SystemTime</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>wMonth</c> field was the following:  <c>unsigned short wMonth</c>;</para></remarks>
            // IDL: unsigned short wMonth;
            public ushort wMonth;

            /// <summary><para><c>wDayOfWeek</c> field of <c>SystemTime</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>wDayOfWeek</c> field was the following:  <c>unsigned short wDayOfWeek</c>;</para></remarks>
            // IDL: unsigned short wDayOfWeek;
            public ushort wDayOfWeek;

            /// <summary><para><c>wDay</c> field of <c>SystemTime</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>wDay</c> field was the following:  <c>unsigned short wDay</c>;</para></remarks>
            // IDL: unsigned short wDay;
            public ushort wDay;

            /// <summary><para><c>wHour</c> field of <c>SystemTime</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>wHour</c> field was the following:  <c>unsigned short wHour</c>;</para></remarks>
            // IDL: unsigned short wHour;
            public ushort wHour;

            /// <summary><para><c>wMinute</c> field of <c>SystemTime</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>wMinute</c> field was the following:  <c>unsigned short wMinute</c>;</para></remarks>
            // IDL: unsigned short wMinute;
            public ushort wMinute;

            /// <summary><para><c>wSecond</c> field of <c>SystemTime</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>wSecond</c> field was the following:  <c>unsigned short wSecond</c>;</para></remarks>
            // IDL: unsigned short wSecond;
            public ushort wSecond;

            /// <summary><para><c>wMilliseconds</c> field of <c>SystemTime</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>wMilliseconds</c> field was the following:  <c>unsigned short wMilliseconds</c>;</para></remarks>
            // IDL: unsigned short wMilliseconds;
            public ushort wMilliseconds;
        }

        #endregion

        #region Member Variables

        private IntPtr  m_hBcs;						// Handle to the scanner object.
        private int     m_lcid;                     // Locale identifier (LCID) paramter to native code.
        private uint    m_result;				    // Holds HRESULT from last SDK function call.

        #endregion

        #region Properties

        public string BCS_SDK_VERSION
        {
            get
            {
                return BCS_FILE_VERSION;
            }
        }

        #endregion

		/// <summary>
		/// 
		/// </summary>
		public Scanner()
		{
			m_hBcs		= IntPtr.Zero;				// Set initial value to NULL.
            m_lcid      = System.Threading.Thread.CurrentThread.CurrentUICulture.LCID; 
		}

		#region Connection Functions
#if false
		// This structure holds either a Bluetooth address or a COM port number
		public struct ConnectionAddress
		{											// Identical to BLUETOOTH_ADDRESS
//			public ulong ullLong;			        // Easier to compare against BLUETOOTH_NULL_ADDRESS
			[MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = 6)]
            public byte[] bytes;					// Easier to format
		}
#else
        /// <summary><para><c>ConnectionAddress</c> This structure holds either a Bluetooth address or a COM port number.  It is identical to the Windows BLUETOOTH_ADDRESS structure.</para></summary>
        [StructLayout(LayoutKind.Explicit)]
        public struct ConnectionAddress // UNION
        {
            /// <summary><para><c>value</c> field of <c>ConnectionAddress</c> Easier to compare against BLUETOOTH_NULL_ADDRESS</para></summary>
            /// <remarks><para>An original IDL definition of <c>value</c> field was the following:  <c>unsigned __int64 ullLong</c>;</para></remarks>
            // IDL: unsigned __int64 ullLong;
            [FieldOffset(0)]
            public ulong value;
#if false
            /// <summary><para><c>bytes</c> field of <c>ConnectionAddress</c> Easier to format</para></summary>
            /// <remarks><para>An original IDL definition of <c>bytes</c> field was the following:  <c>unsigned byte[6] rgBytes</c>;</para></remarks>
            // IDL: unsigned byte[6] rgBytes;
            [FieldOffset(0)]
//            [MarshalAs(UnmanagedType.LPArray, SizeConst = 6)]
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 6)]
            public byte[] bytes;
#endif
        }
#endif

        // This value is used in place of a Bluetooth "Class of Device" to indicate that a ConnectionInfo contains serial connection information
        /// <summary><para><c>COD_SERIAL_PORT</c> Indicates that the ConnectionInfo structure is for a serial connection.</para></summary>
        public const uint COD_SERIAL_PORT = (0xffffffff);	// Indicates that the ConnectionInfo structure is for a serial connection
		// This value is the class of device reported by the Flic and Rov Bluetooth battery doors
        /// <summary><para><c>COD_BLUETOOTH_DOOR</c> Indicates that the ConnectionInfo structure is for a Bluetooth connection.</para></summary>
        public const uint COD_BLUETOOTH_DOOR = (0x00000500);	// Indicates that the ConnectionInfo structure is for a Bluetooth connection

		// These values are used to subdivide the szName field of a BLUETOOTH_DEVICE_INFO structure
		// to hold serial connection information
		private const int	BCS_MAX_NAME_SIZE		= 128;			// 127 Unicode characters + NUL (i.e. "Microvision Scanner USB Interface")
		private const int	BCS_MAX_PORTNAME_SIZE	= 8;			//   7 Unicode characters + NUL (i.e. "COM20")
		private const int	BCS_MAX_INSTANCEID_SIZE	= 112;			// 111 Unicode characters + NUL (i.e. "HID\VID_041F&PID_0104\6&31EA2B5D&0&0000")

#if false
		/// <summary>
		/// </summary>
//		[StructLayout(LayoutKind.Sequential, Pack=1)]
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        public struct ConnectionInfo
		{
			public uint         size;				// Size, in bytes, of this structure - must be the sizeof BLUETOOTH_DEVICE_INFO
			//[MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst=LEN_BT_DEVICE_ADDRESS)]
			public ConnectionAddress address;		// Bluetooth address or COM port number
			public uint	        classofDevice;	    // Bluetooth "Class of Device" or BCS_COD_SERIAL_PORT
			public bool		    connected;			// Device connected/in use
			public bool		    remembered;		    // Device remembered
			public bool		    authenticated;		// Device authenticated/paired/bonded
			public SystemTime   lastSeen;			// Last time the device was seen
			public SystemTime   lastUsed;			// Last time the device was used for other than RNR, inquiry, or SDP
			[MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = BCS_MAX_NAME_SIZE)]
            public string       name;			    // Name of the device.

			[MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = BCS_MAX_PORTNAME_SIZE)]
			public byte[]       portName;			// PortName of the serial device
			[MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = BCS_MAX_INSTANCEID_SIZE)]
			public byte[]       instanceId;			// Instance ID of the serial device
		}
#else
        /// <summary><para><c>ConnectionInfo</c> record.</para></summary>
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        public struct ConnectionInfo // RECORD
        {
            /// <summary><para><c>dwSize</c> field of <c>ConnectionInfo</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>dwSize</c> field was the following:  <c>unsigned long dwSize</c>;</para></remarks>
            // IDL: unsigned long dwSize;
            private uint dwSize;

            /// <summary><para><c>address</c> field of <c>ConnectionInfo</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>address</c> field was the following:  <c>ConnectionAddress Address</c>;</para></remarks>
            // IDL: ConnectionAddress Address;
            public ConnectionAddress address;

            /// <summary><para><c>classofDevice</c> field of <c>ConnectionInfo</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>classofDevice</c> field was the following:  <c>unsigned long ulClassofDevice</c>;</para></remarks>
            // IDL: unsigned long ulClassofDevice;
            public uint classofDevice;

            /// <summary><para><c>connected</c> field of <c>ConnectionInfo</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>connected</c> field was the following:  <c>long fConnected</c>;</para></remarks>
            // IDL: long fConnected;
            public int connected;

            /// <summary><para><c>remembered</c> field of <c>ConnectionInfo</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>remembered</c> field was the following:  <c>long fRemembered</c>;</para></remarks>
            // IDL: long fRemembered;
            public int remembered;

            /// <summary><para><c>authenticated</c> field of <c>ConnectionInfo</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>authenticated</c> field was the following:  <c>long fAuthenticated</c>;</para></remarks>
            // IDL: long fAuthenticated;
            public int authenticated;

            /// <summary><para><c>lastSeen</c> field of <c>ConnectionInfo</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>lastSeen</c> field was the following:  <c>SystemTime stLastSeen</c>;</para></remarks>
            // IDL: SystemTime stLastSeen;
            public SystemTime lastSeen;

            /// <summary><para><c>lastUsed</c> field of <c>ConnectionInfo</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>lastUsed</c> field was the following:  <c>SystemTime stLastUsed</c>;</para></remarks>
            // IDL: SystemTime stLastUsed;
            public SystemTime lastUsed;

            /// <summary><para><c>name</c> field of <c>ConnectionInfo</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>name</c> field was the following:  <c>unsigned short[128] szName</c>;</para></remarks>
            // IDL: unsigned short[128] szName;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = BCS_MAX_NAME_SIZE)]
            public string name;

            /// <summary><para><c>portName</c> field of <c>ConnectionInfo</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>portName</c> field was the following:  <c>unsigned short[8] szPortName</c>;</para></remarks>
            // IDL: unsigned short[8] szPortName;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = BCS_MAX_PORTNAME_SIZE)]
            public string portName;

            /// <summary><para><c>instanceId</c> field of <c>ConnectionInfo</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>instanceId</c> field was the following:  <c>unsigned short[112] szInstanceId</c>;</para></remarks>
            // IDL: unsigned short[112] szInstanceId;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = BCS_MAX_INSTANCEID_SIZE)]
            public string instanceId;

            public override string ToString()
            {
                return name;
            }
        }
#endif
        #endregion

#if false
		/// <summary>
		/// 
		/// </summary>
		/// <param name="pvParam"></param>
		/// <param name="connectionInfo"></param>
		/// <returns></returns>
		public delegate uint ConnectionCallback (
			IntPtr				pvParam,
			out ConnectionInfo  connectionInfo);
#endif

        #region Event Functions

        // NOTE: The instance id field can be used to retrieve additional information about a serial device:
//       Use SetupDiCreateDeviceInfoList function to create an empty device information set.
//       Then use SetupDiOpenDeviceInfo to locate a device directly from its device instance id:
//       i.e. SetupDiOpenDeviceInfo( hDevInfo, szDeviceInstanceId, NULL, 0, &DeviceInfoData );

		/// <summary>
		/// 
		/// </summary>
		public enum EventType
		{
			/// <summary>
			/// Banner received from the scanner.
			/// </summary>
			FlicBanner,
			/// <summary>
			/// Data received from the scanner.
			/// </summary>
			DataReceived,
			/// <summary>
			/// Data identified as a bar code (not implemented).
			/// </summary>
			BarCodeDataReceived,
			/// <summary>
			/// Packet that will be sent to the scanner.
			/// </summary>
			CommandPacket,
			/// <summary>
			/// Packet received from the scanner.
			/// </summary>
			ResponsePacket,
			/// <summary>
			/// Attempting to connect with Bluetooth device.
			/// </summary>
			BluetoothAttemptingToConnect,
			/// <summary>
			/// Connection with Bluetooth device lost.
			/// </summary>
			BluetoothConnectionLost,
			/// <summary>
			/// Bluetooth connection established.
			/// </summary>
			BluetoothDeviceConnected
			// Additional event types will be added here.  Applications should return S_OK from the event callback function for any unrecognized event types.
		}
		#endregion
		
		
		#region Data Functions
		private const int	BCS_MAX_DATA_RESPONSE_SIZE	= 16;		// 16 bytes


		/// <summary>
		/// Data packet information structure received through the pEvent parameter of an event
		/// callback when the event type is eDataReceived.
		/// </summary>
		[StructLayout(LayoutKind.Sequential, Pack=1)]
		public struct EventData
		{
			int				dwSize;							// Size, in bytes, of this structure - must be the sizeof EventData
            /// <summary><para><c>pDataReceived</c> field of <c>EventData</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>pDataReceived</c> field was the following:  <c>unsigned byte* pDataReceived</c>;</para></remarks>
            [ComConversionLoss]
            [MarshalAs(UnmanagedType.I4)]
            public IntPtr   pDataReceived;                   // Pointer to the bytes received from the scanner
            /// <summary><para><c>cbDataReceived</c> field of <c>EventData</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>cbDataReceived</c> field was the following:  <c>int cbDataReceived</c>;</para></remarks>
            int cbDataReceived;					 // [in] Count of bytes received from the scanner, and [out] number of bytes to discard
            /// <summary><para><c>rgResponse</c> field of <c>EventData</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>rgResponse</c> field was the following:  <c>unsigned byte[16] rgResponse</c>;</para></remarks>
            [MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = BCS_MAX_DATA_RESPONSE_SIZE)]
			public byte[]   rgResponse;						//Bluetooth device address.
            /// <summary><para><c>cbResponse</c> field of <c>EventData</c> record.</para></summary>
            /// <remarks><para>An original IDL definition of <c>cbResponse</c> field was the following:  <c>int cbResponse</c>;</para></remarks>
            public int cbResponse;						// Number of bytes to send in response
		}

		/// <summary>
		/// These values are bitwise or'd together to describe how to proceed from an event
		/// callback.
		/// </summary>
		public enum EventDataResponses
		{
			/// <summary>
			/// Discard cbData bytes and continue processing (Same as S_OK).
			/// </summary>
			DiscardData	= 0x00,
			/// <summary>
			/// Do not discard any bytes and continue reading from port.
			/// </summary>
			NeedMoreData = 0x01,
			/// <summary>
			/// Send cbResponse bytes from rgResponse as a response and continue processing.
			/// </summary>
			ReplyToScanner = 0x02,
		}
		
#if false
		/// <summary>
		/// Discard cbData bytes and continue processing (Same as S_OK).
		/// </summary>
		public const int DATA_DISCARD		= 0x00;
		/// <summary>
		/// Do not discard any bytes and continue reading from port.
		/// </summary>
		public const int DATA_MORE = 0x01;
		/// <summary>
		/// Send cbResponse bytes from rgResponse as a response and continue processing.
		/// </summary>
		public const int DATA_REPLY			= 0x02;
		/// <summary>
		/// Bit mask of valid EventData bits.
		/// </summary>
		public const int DATA_MASK			= 0x03;
#endif
		#endregion
		

		#region Connection Management Functions
		// These values are bitwise or'd together to determine the behavior of the BcsEnumConnections function
		/// <summary>
		/// Show potential serial connections.
		/// </summary>
		public const int ENUM_SERIAL		= 0x01;
		/// <summary>
		/// Show Bluetooth devices with a Class of Device of 0x500 (COD of Flic).
		/// </summary>
		public const int ENUM_BLUETOOTH		= 0x02;
		/// <summary>
		/// Issue an inquiry to discover new Bluetooth devices.
		/// </summary>
		public const int ENUM_DISCOVER		= 0x04;
		/// <summary>
		/// Call BluetoothSelectDevices or equivalent to discover Bluetooth using a common dialog.
		/// </summary>
		public const int ENUM_SELECT		= 0x08;
		/// <summary>
		/// Append Bluetooth Device Address (BDA) if device has default name (i.e. "Cordless").
		/// </summary>
		public const int ENUM_APPEND_BDA	= 0x10;
		/// <summary>
		/// Return serial device names using "friendly" (SPDRP_FRIENDLYNAME) format (Appends " (COM#)").
		/// </summary>
		public const int ENUM_FRIENDLYNAME	= 0x20;
		/// <summary>
		/// Bit mask of valid BCS_ENUM bits.
		/// </summary>
		public const int ENUM_MASK			= 0x3f;

// BcsOpenConnection (dwBaudRate and pdwBaudRate)
		/// <summary>
		/// Unknown (or not applicable) baud rate (sets BCS_OPEN_DETECT_BAUDRATE flag bit for serial connections).
		/// </summary>
		public const int UNKNOWN_BAUDRATE	= 0;

// These values are bitwise or'd together to determine the behavior of the BcsOpenConnection function
		/// <summary>
		/// Detect the existence of a scanner.
		/// </summary>
		public const int OPEN_DETECT_SCANNER	= 0x01;
		/// <summary>
		/// Auto detect the baud rate. The dwBaudRate parameter is treated as a hint.
		/// </summary>
		public const int OPEN_DETECT_BAUDRATE	= 0x02;
		/// <summary>
		/// Fail to open the port and return an error code if BCS_OPEN_DETECT_SCANNER fails.
		/// </summary>
		public const int OPEN_FAIL_NO_DETECT	= 0x04;
		/// <summary>
		/// Get the Microvision banner from the scanner and send it via the event callback function.
		/// </summary>
		public const int OPEN_GET_BANNER		= 0x08;
		/// <summary>
		/// Generate an event for each M2M packet.
		/// </summary>
		public const int OPEN_PACKET_EVENTS		= 0x10;
		/// <summary>
		/// Bit mask of valid BCS_OPEN bits.
		/// </summary>
		public const int OPEN_MASK				= 0x1f;

//--------------------- Connection Management Functions ----------------------

        /// <summary>
        /// 
        /// </summary>
        /// <param name="pvParam"></param>
        /// <param name="connectionInfo"></param>
        /// <returns></returns>
        public delegate int ConnectionCallback(
			IntPtr				pvParam,
            ref ConnectionInfo  connectionInfo);

        /// <summary><para><c>BcsEnumConnections</c> </para><para>Calls a callback function for each potential connection</para></summary>
        [DllImport(DLL_FILENAME, PreserveSig = false)]
        public static extern uint BcsEnumConnections(
            //ref Guid a_pGuidClass,
            int                 guid,
            IntPtr              hwndParent,		// ??? or IWin32Window
			ConnectionCallback	pfnConnectionCallback,
			IntPtr	        	pvParam,
			uint				dwFlags);
		/// <summary>
		/// Calls a callback function for each potential connection
		/// </summary>
		/// <param name="hwndParent"></param>
		/// <param name="connectionCallback"></param>
		/// <param name="flags"></param>
		/// <returns></returns>
		public void EnumConnections(
			IntPtr				hwndParent,			// IN  Window handle to be used as the parent window for any dialogs displayed by this function
			ConnectionCallback connectionCallback,  // IN A callback that will be called for each Connection.
			uint				flags)			    // IN  Bitwise or'd combination of BCS_ENUM bits
		{
			Guid pGuidClass = Guid.Empty;
			m_result = BcsEnumConnections (
				//Guid.Empty, /* a_pGuidClass, */
                //ref Guid.Empty,					// (IN) Reserved, must be NULL.
                0,
                hwndParent,
				connectionCallback,
				IntPtr.Zero,
				flags);
		}

        [DllImport(DLL_FILENAME, PreserveSig = false)]
        private static extern uint BcsDisplayConnectionProperties(
			IntPtr				hwndParent,			// ??? or IWin32Window
			ref ConnectionInfo  pConnectionInfo,
			uint				dwFlags);
		/// <summary>
		/// Invokes the Control Panel's device information property sheet.
		/// </summary>
		/// <param name="hwndParent"></param>
		/// <param name="connectionInfo"></param>
		/// <returns></returns>
		public void DisplayConnectionProperties(
			IntPtr				hwndParent,			// IN  Window handle to be used as the parent window for dialog displayed by this function
			ConnectionInfo	    connectionInfo)	    // IN  A connection information structure (usually retrieved via BcsEnumConnections)
		{
			m_result = BcsDisplayConnectionProperties(hwndParent, ref connectionInfo, 0);
		}


        [DllImport(DLL_FILENAME, PreserveSig = false)]
        private static extern uint BcsRemoveConnection(
			ref ConnectionInfo  pConnectionInfo,
			uint				dwFlags);
		/// ---------------------------------------------------------------------------------------
		/// <summary>
		/// Removes authentication between a Bluetooth device and the computer, purging any cached
		/// information about the device.
		/// </summary>
		/// <param name="connectionInfo">[IN] The connection info for the connection to remove.</param>
		/// <returns>
		/// </returns>
		/// ---------------------------------------------------------------------------------------
		public void RemoveConnection (
			ConnectionInfo connectionInfo)
		{
			m_result = BcsRemoveConnection(ref connectionInfo, 0);
		}


        [DllImport(DLL_FILENAME, PreserveSig = false)]
        private static extern uint BcsOpenConnection(
			out	IntPtr			hBcs,
			ref ConnectionInfo  pConnectionInfo,
			uint				dwBaudRate,
			out uint			pdwBaudRate,
			EventCallback		pfnEventCallback,
			IntPtr		        pvParam,
			uint				dwFlags);
		/// <summary>
		/// Opens a connection using a connection information structure.
		/// </summary>
		/// <param name="connectionInfo"></param>
		/// <param name="baudRate"></param>
		/// <param name="eventCallback"></param>
		/// <param name="flags"></param>
        /// <returns>Baud rate of the connection.</returns>
		public uint OpenConnection(
			ConnectionInfo	    connectionInfo,		// IN  A connection information structure (usually retrieved via BcsEnumConnections)
			uint				baudRate,			// IN  A specific baud rate (4800|9600|19200|38400|57600|115200) or BCS_UNKNOWN_BAUDRATE
			EventCallback		eventCallback,		// IN  A callback that will be called for each event.
			uint				flags)				// IN  Bitwise or'd combination of BCS_OPEN bits or 0
		{
            uint outBaudRate;
			m_result = BcsOpenConnection(
				out m_hBcs,
				ref connectionInfo,
				baudRate,
                out outBaudRate,
				eventCallback,
				IntPtr.Zero,
				flags);
            return outBaudRate;
		}


        [DllImport(DLL_FILENAME, PreserveSig = false)]
        private static extern uint BcsOpenPort(
			out IntPtr		phBcs,
			string			pszPort,
            uint            ulClassofDevice,
			uint			dwBaudRate,
			out uint		pdwBaudRate,
			EventCallback   pfnEventCallback,
            IntPtr          pvParam,    // IN  Parameter to be passed to pfnEventCallback as the pvParam
			uint			dwFlags);
		/// <summary>
		/// Creates a connection.
		/// </summary>
		/// <param name="port">          IN  String that contains a port name or Bluetooth address hexadecimal string.</param>
		/// <param name="classofDevice"> IN  Bluetooth "Class of Device" or BCS_COD_SERIAL_PORT.</param>
		/// <param name="baudRate">      IN  A specific baud rate (4800|9600|19200|38400|57600|115200) or BCS_UNKNOWN_BAUDRATE.</param>
		/// <param name="eventCallback"> IN  A callback that will be called for each event.</param>
		/// <param name="flags">         IN  Bitwise or'd combination of BCS_OPEN bits or 0</param>
		/// <returns></returns>
		public uint OpenPort (
			string			port,
			uint			classofDevice,
			uint			baudRate,
			EventCallback	eventCallback,
			uint			flags)
		{
            uint outBaudRate;
			m_result = BcsOpenPort(
				out m_hBcs, 
				port,
				classofDevice, 
				baudRate,
				out outBaudRate, 
				eventCallback,
			    IntPtr.Zero,
				flags);
            return outBaudRate;
		}       // end of OpenPort


        [DllImport(DLL_FILENAME, PreserveSig = false)]
        private static extern uint BcsIsConnected(IntPtr hBcs, uint dwFlags);
		/// ---------------------------------------------------------------------------------------
		/// <summary>
		/// Returns the connection status.
		/// </summary>
		/// <returns></returns>
		/// ---------------------------------------------------------------------------------------
        public bool IsConnected()   // MWB !!! Should name be "Connected"?  Should Connected be a property rather than a method?
		{
            if (m_hBcs == IntPtr.Zero)
                return false;

            const uint S_OK = 0;
			return BcsIsConnected(m_hBcs, 0) == S_OK;
		}


        [DllImport(DLL_FILENAME, PreserveSig = false)]
        private static extern uint BcsCloseConnection (IntPtr a_hBcs);
		/// ---------------------------------------------------------------------------------------
		/// <summary>
		/// Call .CloseConnection to close the scanner connection. Calling this method makes the
		/// COM port available for other uses. It is not necessary to call .CloseConnection to
		/// close the port but it IS good programming practice. The wrapper automatically closes
		/// the port when destructing the object if you do not explicitly close it.
		/// </summary>
		/// <returns></returns>
		/// ---------------------------------------------------------------------------------------
		public void CloseConnection()
		{
			m_result = BcsCloseConnection(m_hBcs);
            m_hBcs = IntPtr.Zero;
		}
		#endregion


		/// <summary>
		/// Maximum length of a bar code.
		/// </summary>
		public const int MAX_BARCODE_SIZE	= 96;
		

        [DllImport(DLL_FILENAME, PreserveSig = false)]
        private static extern uint BcsGetBarCodeCount (IntPtr a_hBcs, out int nCount);
		/// ---------------------------------------------------------------------------------------
		/// <summary>
		/// GetBarCodeCount returns the number of bar codes currently residing in the API memory. You
		/// can also use this function to detect the status of a download.
		/// </summary>
		/// <returns>
        /// The number of bar codes stored in the scanner.
		/// </returns>
		/// ---------------------------------------------------------------------------------------
		public int GetBarCodeCount()
		{
            int count;
            m_result = BcsGetBarCodeCount(m_hBcs, out count);
            return count;
		}

#if false
		// Bar code download information structures
		struct DataAttributes 					        // Parsed data element attributes
		{
			uint	size;							    // Size, in bytes, of this structure
			string	deviceSn;						    // 'dev'	Device Serial Number
			int	    packetId;						    // 'id'		Download packet identifier
			int	    barCodeCount;					    // 'bcc'	Number of bar codees stored in the scanner at the time the download data packet was sent
			string	downloadTimestampString;		    // 'dt'		Value of scanners internal clock at time of download (19 chars + NUL).  Only valid if Timestamp = True.
			object	downloadTimestamp;      			// 'dt'		Value of scanners internal clock at time of download expressed as a VARIANT.  DATE type if conversion possible.
			int	    batteryEventCount;				    // 'ec'		Battery event count (e# for next bar code).  Missing value indicated by -1.
		}

		struct BarCodeAttributes						// Parsed bc element attributes
		{
            uint    size;							    // Size, in bytes, of this structure
			string	symIdType;						    // 'si'		Symbology identifier type ("Off", "AIM", or "Custom")
			string	timestamp;						    // 'ts'		Timestamp
			string	timeType;						    // 'tt'		Time type (OK, or eN - where N = number of power cycles since "Set Time")
		}

		struct DownloadInfo
		{
            uint                    size;				// Size, in bytes, of this structure
//			DataAttributes *        dataAttributes;	    // Pointer to data element attributes structure
            [ComConversionLoss]
            [MarshalAs(UnmanagedType.I4)]
            public IntPtr           pDataAttributes;
//            BarCodeAttributes*    barCodeAttributes;	// Pointer to bar code element attributes structure
            [ComConversionLoss]
            [MarshalAs(UnmanagedType.I4)]
            public IntPtr           pBcAttribs;
            IntPtr                  reserved;		    // Reserved for internal use
		}
#endif

		/// <summary>
		/// Pass this value for the nDelPacketId parameter of BcsGetBarCodes to indicate that
		/// there is no packet to delete.
		/// </summary>
		public const int BC_NO_PACKET_ID= -1;

		/// <summary>
		/// Pass this value for the nMaxBarCodes parameter of BcsGetBarCodes to get all available
		/// bar codes via the callback function.
		/// </summary>
		public const int BC_GET_ALL		= -1;

		// These values are bitwise or'd together determine the behavior of the BcsGetBarCodes function
		/// <summary>
		/// Automatically delete all bar codes if BcsGetBarCodes does not return an error.
		/// </summary>
		public const int BC_AUTO_DELETE			= 0x01;
		/// <summary>
		/// Adjust certain timestamp values (pszTimeStamp) and change pszTypeType accordingly.  See documentation for details.
		/// </summary>
		public const int BC_ADJUST_TIMESTAMPS	= 0x02;
		/// <summary>
		/// Bit mask of valid BCS_BC bits.
		/// </summary>
		public const int BC_MASK				= 0x03;


        [DllImport(DLL_FILENAME, PreserveSig = false)]
        private static extern uint BcsGetBarCodes(
			IntPtr				hBcs,
			int				    nDelPacketId,
			int				    nMaxBarCodes,
			BarCodeCallback	pfnBarCodeCallback,
			IntPtr		        pvParam,            // [IN ] Parameter to be passed to pfnBarCodeCallback as the pvParam.
			out int			    pDelPacketId,
			out int			    pRemainingBarCodes,
			uint				dwFlags);
		///--------------------------------------------------------------------------------------
		///
		/// <summary>
		/// Send a download request to the scanner. If the scanner contains bar codes then the
		/// bar codes are sent in a batch and a callback is activated for each bar code received.
		/// This function returns after the callback function has been called for all bar codes
		/// in the current batch.
		/// </summary>
        /// <param name="deletePacketId">
		/// [IN ] Packet identifier of packet to delete (use BCS_BC_NO_PACKET_ID to NOT delete).
        /// </param>
		/// <param name="maxBarCodes">
		/// [IN ] Maximum number of bar codes to retrieve or BCS_BC_GET_ALL to get all available
		/// bar codes.
        /// </param>
		/// <param name="barCodeCallback">
		/// [IN ] A callback that is called for each bar code downloaded in this batch.
		/// If the callback returns >= 0, the enumeration continues.
		/// If the callback returns less than 0, the enumeration stops and the value is returned.
        /// </param>
		/// <param name="remainingBarCodes">
		/// [OUT] Pointer to the number of bar codes remaining in the scanner after the host
		/// receives this batch of bar codes.
        /// </param>
		/// <param name="flags">
		/// [IN ] 0 or BCS_BC_AUTO_DELETE. If zero then the received bar codes are retained in
		/// scanner memory until the host specifically deletes them. With BCS_BC_AUTO_DELETE the
		/// entire batch of bar codes is deleted after each has been successfully received.
        ///</param>
        /// <returns>
        /// The packet identifier of the current packet. This is often passed as the
		/// nDelPacketId parameter on the next call to BcsGetBarCodes (to delete the previous
		/// packet).
        /// </returns>
		///--------------------------------------------------------------------------------------
		public int GetBarCodes (
			int				    deletePacketId,
			int				    maxBarCodes,
			BarCodeCallback	barCodeCallback,
			out int			    remainingBarCodes,
			uint				flags)
		{
            int outDeletePacketId;
            m_result = BcsGetBarCodes(
				m_hBcs,
                deletePacketId,
                maxBarCodes,
                barCodeCallback,
                IntPtr.Zero,
                out outDeletePacketId,
                out remainingBarCodes,
                flags);
            return outDeletePacketId;
		}


		#region Scanner Management Functions
		///-----Scanner Management Functions-----------------------------------------------------
		
		public const int MAX_XSP_COMMAND= 224;	// Currently limited by Bluetooth packet size and packet overhead

		// These values are bitwise or'd together determine the behavior of the Bcs Scanner Management functions (NOTE:  May not support BCS_XSP_BSTR in release code)
		// NOTE:  Actually, the only flags defined so far are all mutually exclusive
		/// <summary>
		/// Return the raw XSP XML packet as a BSTR	(No XML parsing).
		/// </summary>
		public const int XSP_XML	= 0x01;
		/// <summary>
		/// Return the raw XSP reply element as a BSTR	(No type conversion).
		/// </summary>
		public const int XSP_BSTR	= 0x02;
		/// <summary>
		/// Bit mask of valid BCS_XSP bits.
		/// </summary>
		public const int XSP_MASK	= 0x03;


        [DllImport(DLL_FILENAME, PreserveSig = false)]
        private static extern uint BcsCommand(
			IntPtr	    hBcs,
			string	    pszCommand,
			int		    lcid,
			ref object	pVarArg,
			out object	pVarResult,
			uint	    dwFlags);
		///--------------------------------------------------------------------------------------
		///
		/// <summary>
		/// Use BcsCommand to send operational commands to the scanner. The pszCommand parameter
		/// is the XSP command string that is passed through to the XSP layer.  Use BcsCommand to
		/// issue any XSP command including download commands, clear commands, or commands to get
		/// or set scanner properties (also handled by BcsGetProp and BcsSetProp). See the XSP
		/// manual for a complete list of scanner properties.
		/// </summary>
		///
		/// <param name="command" > [IN ] XSP command string.</param>
        /// <param name="argument"> [IN ] Command argument.</param>
		/// <param name="flags"   > [IN ] Accepts 0, BCS_XSP_XML or BCS_XSP_BSTR.</param>
		///
		///--------------------------------------------------------------------------------------
		public object Command( string command, object argument, uint flags )
		{
            object resultObject;
            m_result = BcsCommand(m_hBcs, command, m_lcid, ref argument, out resultObject, flags);
            return resultObject;
		} 


        [DllImport(DLL_FILENAME, PreserveSig = false)]
        private static extern uint BcsSetProp(
			IntPtr	    hBcs,
			string	    pszPropName,
			int		    lcid,
			ref object	pVarValue,
			uint		dwFlags);
		///--------------------------------------------------------------------------------------
		///
		/// <summary>
		/// Use BcsSetProp to set the value of a configuration property in the scanner. The
		/// property parameter must match the XSP Get Command Strings and are passed through to
		/// the XSP layer. See the XSP manual for a complete list of scanner properties.
		/// </summary>
		///
		/// <param name="name" >[IN] String version of property name.</param>
		/// <param name="value">[IN] New propery value.</param>
		///
		///--------------------------------------------------------------------------------------
		public void SetProp( string name, object value )
		{
            m_result = BcsSetProp(m_hBcs, name, m_lcid, ref value, 0);
		}


        [DllImport(DLL_FILENAME, PreserveSig = false)]
        private static extern uint BcsGetProp (
			IntPtr	    hBcs,
			string	    pszPropName,
			int		    lcid,
			out object	pVarResult,
			uint		dwFlags);
		///--------------------------------------------------------------------------------------
		///
		/// <summary>
		/// Use BcsGetProp to read the value of a configuration property in the scanner. The
		/// property parameter must match the XSP Get Command Strings and are passed through to
		/// the XSP layer. See the XSP manual for a complete list of scanner properties.
		/// </summary>
		///
		/// <param name="name">[IN ] Property name string.</param>
        /// <returns>Property value object</returns>
        ///
		///--------------------------------------------------------------------------------------
		public object GetProp( string name )
		{
            object resultObject;
            m_result = BcsGetProp(m_hBcs, name, m_lcid, out resultObject, 0);
            return resultObject;
        }


        [DllImport(DLL_FILENAME, PreserveSig = false)]
        private static extern uint BcsSignal(
			IntPtr	hBcs,
			int	nParamSets,
			int	nFrequency,
			int	nDuration,
			bool	bLed0,
			bool	bLed1 /* ,... */);
		///--------------------------------------------------------------------------------------
		///
		/// <summary>
		/// Does signal stuff.
		/// </summary>
		///
		/// <param name="frequency">[IN] Frequency in kHz, If freq = 0, beeper is silent.</param>
		/// <param name="duration" >[IN] Duration in 0.5 ms intervals.</param>
		/// <param name="led0"     >[IN] Led0 is On (1) or Off (0) during the signal.</param>
		/// <param name="led1"     >[IN] Led1 is On (1) or Off (0) during the signal.</param>
		///
		///--------------------------------------------------------------------------------------
		public void Signal (
			int	frequency,				// IN  Frequency in kHz, If freq = 0, beeper is silent
			int	duration,				// IN  Duration in 0.5 ms intervals
			bool	led0,					// IN  Led0 is On (1) or Off (0) during the signal
			bool	led1 /* ,... */)		// IN  Led1 is On (1) or Off (0) during the signal
		{
			m_result = BcsSignal(m_hBcs, 1, frequency, duration, led0, led1);
		}
		#endregion
		
	//-------------------------------------------------------------------------------------------------
	}
}

