'***************************************************************************************
'
' Simple VB Application Using Microvision .NET API
' Copyright  2007 Microvision, Inc.
' All Rights Reserved.
'
'***************************************************************************************

Imports System.Text
Imports System.Runtime.InteropServices
Imports System.Windows.Forms

Imports Microvision.BarCodeScanner


Public Class BcsBarcodeSampleApp

    ''' <summary>
    ''' Represents an instance of the wrapper of Bcs.dll.
    ''' </summary>
    Dim m_bcsApi As New Scanner()

    ''' <summary>
    ''' Represents an instance of download mode.
    ''' </summary>
    Dim mDownload As DOWNLOAD

#Region "Constants"

    ''' <summary>
    ''' Represents maximum number of characters in the scanner UserID field.
    ''' </summary>
    Private Const LEN_DEVICE_ID As Integer = 16

    ''' <summary>
    ''' Represents the title of messages to be shown to user.
    ''' </summary>
    Private Const MSG_TITLE As String = "BarCode.VB"

#End Region

#Region "Enums"

    ''' <summary>
    ''' These are the possible states of the download mode of scanner data.
    ''' </summary>
    Enum DOWNLOAD

        AUTOMATIC
        MANUAL

    End Enum

    ''' <summary>
    ''' These are the possible states of the application. These values are used by the
    ''' UpdateControls function to set the state of the controls on the main dialog.
    ''' </summary>
    Enum DISPLAY_STATE

        INITIAL
        READY

    End Enum

#End Region

    ''' <summary>
    ''' Represents the method to be called to find new bluetooth scanners.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub btnFind_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnFind.Click

        lblMessage.Text = "Searching for Bluetooth scanners"
        RefreshConnectionList(DISPLAY_STATE.READY, True)

    End Sub

    ''' <summary>
    ''' Represents tyhe method to refreshes the connection list with new connections discovered.
    ''' </summary>
    ''' <param name="displayState">Represents display state.</param>
    ''' <param name="discoverDevicesDialog"></param>
    Private Sub RefreshConnectionList(ByVal displayState As DISPLAY_STATE, ByVal discoverDevicesDialog As Boolean)

        Try

            Dim enumFlags As UInteger = Scanner.ENUM_SERIAL Or Scanner.ENUM_BLUETOOTH Or Scanner.ENUM_APPEND_BDA Or Scanner.ENUM_FRIENDLYNAME

            If discoverDevicesDialog Then

                enumFlags = enumFlags Or Scanner.ENUM_SELECT   ' Select (Discover) Bluetooth devices dialog
            End If

            lstConnection.Items.Clear()         ' The list box will be re-populated by MyConnectionCallback

            Dim connectionCallback As Scanner.ConnectionCallback = AddressOf MyConnectionCallback

            ' Because BcsEnum is intelligent about potential connections, there usually will be very few
            ' scanners to choose from. All possible scanner connections should be enumerated after
            ' returning from the BcsEnumConnections function. The callback should have been called for
            ' each possible connection.

            ' (IN) Handle to parent window.
            ' (IN) callback called for each connection.
            ' (IN) OR'ed ENUM bits.
            m_bcsApi.EnumConnections(Me.Handle, connectionCallback, enumFlags)

            If (lstConnection.Items.Count > 0) Then      ' One or more connections

                lstConnection.SetSelected(0, True)
                lblMessage.Text = "Select connection and press Activate "

            Else

                lblMessage.Text = "No connections found"

            End If
        Catch ex As Exception
            lblMessage.Text = ex.Message

        End Try

        UpdateControls(displayState)

    End Sub

    '''----------------------------------------------------------------------------------------
    ''' <summary>
    ''' This is the enumeration callback function. When the Add function is called then this
    ''' function is called for each port (serial port or USB VCOMM port). It only recognizes
    ''' VCOMM ports when the USB cable is connected. The COM ports detected may or may not
    ''' have a scanner attached. This callback populates the connection list box and a member
    ''' connection array. Activate uses the connection structure stored in the connection list
    ''' to open a connection to a specific scanner.
    ''' </summary>
    ''' <param name="param">[in/out] Generic parameter used to communicate with the function.
    ''' This is not used and a member array is used to save the connection information.</param>
    ''' <param name="connectionInfo">[in] Is loaded with connection details when this
    ''' function is called.</param>
    ''' <returns>
    ''' A value >= 0 if enumeration is to continue; less than 0 if enumeration should stop.
    ''' </returns>
    '''----------------------------------------------------------------------------------------
    Function MyConnectionCallback(ByVal param As IntPtr, ByRef connectionInfo As Scanner.ConnectionInfo) As Integer

        lstConnection.Items.Add(connectionInfo)
        Return 0

    End Function

    ''' -----------------------------------------------------------------------------------------
    ''' <summary>
    ''' This function enables and disables the controls that are appropriate for the current state of
    ''' the application. Handling the controls is critical because many of functions do not contain error
    ''' checking to insure that the control was activated at an appropriate time. The application assumes
    ''' that controls that should be selected are not enabled.
    ''' </summary>
    ''' <param name="displayState">Set controls as appropriate for this operating state.</param>
    '''-----------------------------------------------------------------------------------------
    Sub UpdateControls(ByVal displayState As DISPLAY_STATE)

        Dim bConnectionSelected As Boolean = lstConnection.SelectedIndex >= 0
        Dim bIsConnected As Boolean = m_bcsApi.IsConnected()

        lstConnection.Enabled = Not (bIsConnected)

        btnFind.Enabled = Not (bIsConnected)
        btnRemove.Enabled = Not (bIsConnected) AndAlso bConnectionSelected AndAlso CType(lstConnection.SelectedItem, Scanner.ConnectionInfo).classofDevice = Scanner.COD_BLUETOOTH_DOOR

        btnActivate.Enabled = bConnectionSelected

        radAuto.Enabled = True
        radManual.Enabled = True

        btnSave.Enabled = (editFilename.Text.Length > 0 AndAlso lstOutput.Items.Count > 0)

        btnDownload.Enabled = bIsConnected AndAlso radManual.Checked
        btnClear.Enabled = bIsConnected AndAlso radManual.Checked
        btnGetCount.Enabled = bIsConnected AndAlso radManual.Checked

        btnInitScanner.Enabled = bIsConnected
        grpScannerInfo.Enabled = bIsConnected
        grpDownload.Enabled = bIsConnected
        grpUserId.Enabled = bIsConnected

        Select Case displayState

            Case DISPLAY_STATE.INITIAL
                lblDeviceIdVal.Text = ""
                lblFw.Text = ""
                lblHw.Text = ""
                lblSdk.Text = ""

            Case DISPLAY_STATE.READY
                btnUserId.Enabled = (editUserId.Text.Length > 0)

        End Select

        If (bIsConnected) Then
            btnActivate.Text = "Deactivate"
        Else
            btnActivate.Text = "Activate"
        End If

    End Sub

    ''' <summary>
    ''' Represents the method to be called to activate or deactivate bluetooth connection.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub btnActivate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnActivate.Click

        If (m_bcsApi.IsConnected()) Then     ' Deactivate

            Try

                m_bcsApi.CloseConnection()
                lblMessage.Text = "Scanner connection closed"
                UpdateControls(DISPLAY_STATE.INITIAL)

            Catch ex As Exception

                MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information)

            End Try

        Else

            lblMessage.Text = "Activating connection..."

            OpenConnection(True)       ' Connect to the port

            If (m_bcsApi.IsConnected()) Then

                UpdateControls(DISPLAY_STATE.READY)
                'The scanner was found and the connection was created. Now initialize the scanner. It is 
                'assumed that the scanner does not contain any data at this point. If the scanner does contain
                'data then the time values will not be correct since the initial time is reset.

                If (radAuto.Checked) Then

                    'If we are in auto-download mode then force a download in case the scanner contains
                    'data and was connected prior to the application being active. We don't have to
                    'explicitly read the data because the timer will handle that for us.
                    ReadBarCodeData()

                End If

            Else
                lblMessage.Text = "Connection did not open!"
            End If

        End If
    End Sub

    '''-----------------------------------------------------------------------------------------
    ''' <summary>
    ''' When using auto-download the scanner could contain bar codes and the user may have connected
    ''' the scanner before the application was active. This function starts a download to handle
    ''' this initial case. If a download starts then the event handler generates a data event that 
    ''' will trigger the application to read the bar codes.
    ''' NOTE: The connection to the scanner MUST be open before calling this function.
    ''' </summary>
    ''' <returns>
    ''' true = scanner downloaded data, false = no scanner connected or not data in the scanner.
    ''' </returns>
    '''-----------------------------------------------------------------------------------------
    Function ReadBarCodeData() As Boolean

        Dim dataWasDownloaded As Boolean = False

        'Send a download request to the scanner.
        Return dataWasDownloaded

    End Function


    Dim eventCallback As Microvision.BarCodeScanner.EventCallback

    '''-----------------------------------------------------------------------------------------
    ''' <summary>
    ''' This function creates a connection to the scanner. The scanner must be connected
    ''' to the host or be in range with the Bluetooth radio on. Normally you enumerate a list
    ''' of possible scanners and then select the desired scanner from the list. You can also
    ''' connect to the first scanner found or use some other algorithm of your choosing to
    ''' connect to a scanner.
    ''' After connecting you can either monitor system events (bar code events) or you can manually
    ''' request a data download.
    ''' </summary>
    ''' <param name="connectPrompt">TRUE prompts the user to plug in the scanner, FALSE displays nothing.</param>
    ''' <returns>
    ''' TRUE if the scanner was found and the connection has been opened.
    ''' </returns>
    '''-----------------------------------------------------------------------------------------
    Function OpenConnection(ByVal connectPrompt As Boolean) As Boolean

        Dim baudRate As UInteger

        eventCallback = AddressOf MyEventCallback
        Dim connectionInfo As Scanner.ConnectionInfo = CType(lstConnection.SelectedItem, Scanner.ConnectionInfo)

        ' If we are searching for the scanner then prompt the user to connect the scanner.
        If (connectPrompt) Then

            If (connectionInfo.classofDevice = Scanner.COD_SERIAL_PORT) Then
                MessageBox.Show("Connect scanner", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning)
            Else

                If (Windows.Forms.DialogResult.Cancel = MessageBox.Show("Put scanner in discoverable mode", "Discoverable", _
                        MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1)) Then

                    Return False

                End If

            End If

        End If

        'This next section opens a connection to the scanner. Note that this only opens a
        'connection and it does NOT configure the scanner. Your application should always
        'include some means of configuring the scanner. The scanner configuration is stored
        'in non-volatile memory but you must configure each new scanner once and it is also
        'possible for users to tamper with the scanner configuration. Double-click the
        '"Initialize Scanner" button on the main form to view the initialization function.
        Try

            baudRate = m_bcsApi.OpenConnection(connectionInfo, Scanner.UNKNOWN_BAUDRATE, eventCallback, 0)

            lblMessage.Text = "Scanner connection established"
            Return True

        Catch ex As Exception

            lblMessage.Text = "Scanner connection failed"
            MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information)
        End Try
        Return False

    End Function

    '''-----------------------------------------------------------------------------------------
    ''' <summary>
    ''' Callback function that handles scanner connection events.
    ''' </summary>
    ''' <param name="pvParam"></param>
    ''' <param name="eventType"></param>
    ''' <param name="pEvent"></param>
    ''' <returns></returns>
    '''-----------------------------------------------------------------------------------------
    Function MyEventCallback(ByVal pvParam As IntPtr, ByVal eventType As Scanner.EventType, _
                             ByVal pEvent As IntPtr) As UInteger

        If (radAuto.Checked) Then
            Me.BeginInvoke(New EventHandler(AddressOf DataEventReceived))
        End If
        Return 0

    End Function

    ''' <summary>
    ''' Represents the method to be called to get bar code data.
    ''' </summary>
    ''' <param name="s">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Sub DataEventReceived(ByVal s As Object, ByVal e As EventArgs)

        lblMessage.Text = "Getting Bar Codes"
        lblMessage.Refresh()

        Dim remainingBarCodes As Integer

        Dim barCodeCallback As Scanner.BarCodeCallback = New Scanner.BarCodeCallback(AddressOf MyBarCodeCallback)

        Try

            If (radManual.Checked) Then

                m_bcsApi.GetBarCodes(Scanner.BC_NO_PACKET_ID, Scanner.BC_GET_ALL, barCodeCallback, _
                remainingBarCodes, 0)
            Else
                m_bcsApi.GetBarCodes(Scanner.BC_NO_PACKET_ID, Scanner.BC_GET_ALL, barCodeCallback, _
                remainingBarCodes, Scanner.BC_AUTO_DELETE)

            End If
            lblMessage.Text = "GetBarCodes succeeded"

        Catch ex As Exception
            lblMessage.Text = "GetBarCodes failed"
            MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information)
        End Try

    End Sub

    ''' <summary>
    ''' Represents the method to be called to add the obtained bar code data to a list box.
    ''' </summary>
    ''' <param name="pvParam">Parameter passed into BcsGetBarCodesA as the pvParam</param>
    ''' <param name="symbology">Symbol Identifier (This value is dependent on the setting of the SymIdType property)</param>
    ''' <param name="timeStamp">Timestamp (This pointer value is only valid if Timestamp = True, otherwise the the pointer value is NULL)</param>
    ''' <param name="timeType">Time Type (This pointer value is only valid if Timestamp = True, otherwise the the pointer value is NULL).
    ''' Possible Values: "ok" = Actual Time, or e1, e2, ..., eN = Estimated time after N battery events)</param>
    ''' <param name="barCode">Bar code data</param>
    ''' <param name="downloadInfo">Length of the bar code data in characters</param>
    ''' <returns></returns>
    Function MyBarCodeCallback(ByVal pvParam As IntPtr, ByVal symbology As String, ByVal timeStamp As String, _
                               ByVal timeType As String, ByVal barCode As String, _
                               ByVal cchBarCode As Integer, ByVal downloadInfo As Integer) As UInteger

        lstOutput.Items.Add(FormatOutputLine(barCode, timeStamp).ToString())
        Return 0

    End Function

    ''' <summary>
    ''' Format the bar code, date stamp, and time stamp (if selected). Use the settings in the
    ''' combo boxes for the date/time format.
    ''' </summary>
    ''' <param name="aBarcode">Bar code data read from the scanner.</param>
    ''' <returns>
    ''' The formatted string with bar code, date, and time.
    ''' </returns>
    Function FormatOutputLine(ByVal aBarcode As String, ByVal timeString As String) As StringBuilder

        Dim formatted As StringBuilder = New StringBuilder()
        formatted.Append(aBarcode)

        If (Not String.IsNullOrEmpty(timeString)) Then

            Dim timeStamp As Date = Date.Parse(timeString)

            Select Case cboDate.Text

                Case "mm/dd/yy"
                    formatted.Append(",")
                    formatted.Append(timeStamp.ToString("MM/dd/yy"))

                Case "mm/dd/yyyy"
                    formatted.Append(",")
                    formatted.Append(timeStamp.ToString("MM/dd/yyyy"))

                Case "dd/mm/yy"
                    formatted.Append(",")
                    formatted.Append(timeStamp.ToString("dd/MM/yy"))

                Case "dd/mm/yyyy"
                    formatted.Append(",")
                    formatted.Append(timeStamp.ToString("dd/MM/yyyy"))

            End Select

            Select Case cboTime.Text

                Case "hh:mm"
                    formatted.Append(",")
                    formatted.Append(timeStamp.ToString("hh:mm"))

                Case "hh:mm:ss"
                    formatted.Append(",")
                    formatted.Append(timeStamp.ToString("hh:mm:ss"))

            End Select

        End If

        Return formatted

    End Function

    ''' <summary>
    ''' The User ID button should only be enabled if there is data in the User ID field. When
    ''' the user presses this button then write the User ID text to the scanner.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub btnUserId_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnUserId.Click
        Try

            m_bcsApi.SetProp("UserId", editUserId.Text)
            lblMessage.Text = "User ID set"

        Catch ex As Exception

            lblMessage.Text = "User ID not set"
            MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information)

        End Try

    End Sub

    ''' <summary>
    ''' This function saves the downloaded scanner data in the output list box to the specified file. The
    ''' SAVE button should not be enabled unless there is data in the filename edit box. After writing
    ''' the data to the file clear the output list box.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub btnSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSave.Click
        Try

            If (lstOutput.Items.Count > 0) Then

                Dim outFile As IO.StreamWriter
                outFile = New IO.StreamWriter(editFilename.Text, True, Encoding.ASCII)

                Dim i As Integer
                For i = 0 To lstOutput.Items.Count - 1
                    lstOutput.SelectedIndex = i
                    outFile.WriteLine(lstOutput.SelectedItem)
                Next

                outFile.WriteLine()
                outFile.WriteLine()

                outFile.Close()
                lblMessage.Text = "List data saved to file"
                lstOutput.Items.Clear()

            End If

        Catch ex As Exception

            MessageBox.Show("File Error: " + ex.ToString(), MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information)

        End Try
    End Sub

    ''' <summary>
    ''' This function removes a Bluetooth connection from the connection list.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub btnRemove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRemove.Click
        Try

            m_bcsApi.RemoveConnection(CType(lstConnection.SelectedItem, Scanner.ConnectionInfo))
            RefreshConnectionList(DISPLAY_STATE.READY, False)

        Catch ex As Exception
            lblMessage.Text = ex.Message
        End Try
    End Sub

    ''' <summary>
    ''' Represents the method to be called when download button is clicked to get bar code data from the scanner.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub btnDownload_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDownload.Click
        Dim remainingBarCodes As Integer

        Dim barCodeCallback As Scanner.BarCodeCallback = New Scanner.BarCodeCallback(AddressOf MyBarCodeCallback)

        Try
            If (radManual.Checked) Then

                m_bcsApi.GetBarCodes(Scanner.BC_NO_PACKET_ID, Scanner.BC_GET_ALL, barCodeCallback, _
                remainingBarCodes, 0)
            Else
                m_bcsApi.GetBarCodes(Scanner.BC_NO_PACKET_ID, Scanner.BC_GET_ALL, barCodeCallback, _
                remainingBarCodes, Scanner.BC_AUTO_DELETE)

            End If

            lblMessage.Text = "GetBarCodes succeeded"

        Catch ex As Exception
            lblMessage.Text = "GetBarCodes failed"
            MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information)
        End Try
    End Sub

    ''' <summary>
    ''' This function sends a command instructing the scanner to clear any bar codes buffered in the
    ''' memory of the scanner. This command is normally not used in a live application since most
    ''' applications read the data in blocks and delete each block after it is received and verified.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub btnClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClear.Click

        Try

            ' Parameters to be passed are, 
            ' XSP command,Command argument -- no arg for Clear command and API flag field.
            m_bcsApi.Command("ClearBarCodes", Nothing, 0)

            ' After clearing the bar codes it is a very good idea to synchronize the initial time
            ' value in the scanner with the host computer's clock.
            m_bcsApi.SetProp("Time", System.DateTime.Now)
            lblMessage.Text = "Bar codes cleared!"

        Catch ex As Exception

            lblMessage.Text = "Error clearing bar codes!"
            MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information)
        End Try
    End Sub

    ''' <summary>
    ''' Represents the method to be called when Get Bar Code Count button is 
    ''' clicked to get count of bar codes in scanner.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub btnGetCount_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGetCount.Click
        Try

            Dim count As Integer = m_bcsApi.GetBarCodeCount()
            lblMessage.Text = count & " Barcodes in the scanner"

        Catch ex As Exception

            lblMessage.Text = "Get barcode count failed"
            MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information)
        End Try
    End Sub

    ''' <summary>
    ''' Represents the method to be called when clear list button is 
    ''' clicked to clear the list box containing bar codes in scanner.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub btnClearList_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClearList.Click

        lstOutput.Items.Clear()
        btnSave.Enabled = False
        lblMessage.Text = "List cleared"

    End Sub

    ''' <summary>
    ''' Your application should always have a way to initialize a scanner. If the user receives a new
    ''' scanner from the factory it must be configured once. This function calls scannerConnectionConfigure
    ''' to open the port and force a scanner initialization. The scanner MUST be connected in order for
    ''' this to work.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub btnInitScanner_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnInitScanner.Click
        Try

            ' The initialize button should not be enabled unless there is a valid connection
            ' but check the connection anyway (the error box below should never be called).
            If (Not m_bcsApi.IsConnected()) Then

                MessageBox.Show("Must open a connection to the scanner!", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning)
                Return
            End If

            ' Your application should have a way to configure the scanner and your
            ' configuration should always start by setting the scanner to the factory
            ' defaults. After the scanner is in the factory state then set the scanner
            ' properties required for your application.
            m_bcsApi.Command("RestoreFactorySettings", Nothing, 0)
            m_bcsApi.SetProp("BCDataAck", True)
            m_bcsApi.SetProp("Autodownload", radAuto.Checked)

            lblMessage.Text = "Scanner configured"

        Catch ex As Exception

            lblMessage.Text = "Error configuring scanner!"
            MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information)

        End Try
    End Sub

    ''' <summary>
    ''' Represents the method called when manual radio button check state changes.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub radManual_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles radManual.CheckedChanged

        UpdateControls(DISPLAY_STATE.READY)

    End Sub

    ''' <summary>
    ''' Represents the method called when auto radio button check state changes.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub radAuto_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles radAuto.CheckedChanged

        UpdateControls(DISPLAY_STATE.READY)

    End Sub

    ''' <summary>
    ''' Represents the method to be called when list box selection changes.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub lstConnection_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstConnection.SelectedIndexChanged

        UpdateControls(DISPLAY_STATE.READY)

    End Sub

    ''' <summary>
    ''' Represents the method to be called when Get Info button is clicked.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub btnGetInfo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGetInfo.Click
        Windows.Forms.Cursor.Current = Cursors.WaitCursor

        ' Set the fields to an initial value and then overwrite these as we read them from
        ' the scanner. If we get these values in the application then that indicates that the
        ' call has an error.
        lblDeviceIdVal.Text = "???"
        lblFw.Text = "???"
        lblHw.Text = "???"
        lblSdk.Text = "???"
        editUserId.Clear()

        Try

            ' Read the device ID string from the scanner and store it in a field on the form. If
            ' the returned string is empty then there was an error communicating with the scanner.
            lblDeviceIdVal.Text = m_bcsApi.GetProp("SystemSN").ToString()
            lblFw.Text = m_bcsApi.GetProp("SwVersion").ToString()
            lblHw.Text = m_bcsApi.GetProp("HwVersion").ToString()
            lblSdk.Text = m_bcsApi.BCS_SDK_VERSION
            editUserId.Text = m_bcsApi.GetProp("UserId").ToString()

        Catch ex As Exception

            MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information)

        End Try

        Windows.Forms.Cursor.Current = Cursors.Default

    End Sub

    ''' <summary>
    ''' Represents the method to be called when the screen get loaded.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub BcsBarcodeSampleApp_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        cboDate.Items.Clear()
        cboDate.Items.Add("mm/dd/yy")
        cboDate.Items.Add("mm/dd/yyyy")
        cboDate.Items.Add("dd/mm/yy")
        cboDate.Items.Add("dd/mm/yyyy")
        cboDate.Items.Add("--none--")
        cboDate.SelectedIndex = 0

        cboTime.Items.Clear()
        cboTime.Items.Add("hh:mm:ss")
        cboTime.Items.Add("hh:mm")
        cboTime.Items.Add("--none--")
        cboTime.SelectedIndex = 0

        editFilename.Clear()                        ' Save data to this file.
        editUserId.MaxLength = LEN_DEVICE_ID        ' Limit user input to max User ID field length.
        editUserId.Clear()
        mDownload = DOWNLOAD.MANUAL                 ' Save in a file later.

        radAuto.Checked = (mDownload = DOWNLOAD.AUTOMATIC)
        radManual.Checked = (mDownload = DOWNLOAD.MANUAL)

        lblSdk.Text = ""

        ' Initialize the connection list.
        RefreshConnectionList(DISPLAY_STATE.INITIAL, False)
    End Sub

    ''' <summary>
    ''' Represents the method to be called when a file name has been entered into the file name text box.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub editFilename_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles editFilename.TextChanged

        btnSave.Enabled = editFilename.Text.Length > 0 AndAlso lstOutput.Items.Count > 0

    End Sub

    ''' <summary>
    ''' If the user enters data in the User ID field then enable the SET button. If the field
    ''' is cleared then disable the button.
    ''' </summary>
    ''' <param name="sender">Represents instance raising the event.</param>
    ''' <param name="e">Represents instance containing event data.</param>
    Private Sub editUserId_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles editUserId.TextChanged

        btnUserId.Enabled = editUserId.Text.Length > 0

    End Sub

    Protected Overrides Sub Finalize()
        MyBase.Finalize()
    End Sub
End Class

