/*------------------------------------------------------------------------*\
   BcsConsole.cpp : Simple Console Application Demonstrating how to
   use the Windows Native C API.

   For command help, type help.
   To exit, type exit.
\*------------------------------------------------------------------------*/

#include "stdafx.h"

#include "..\..\BcsSdk\bcs.h"

/*------------------------------------------------------------------------*\
  CONSTANTS, TYPES
\*------------------------------------------------------------------------*/
const int MAX_CONNECTIONS = 4;

typedef struct {
	int nConnections;
	BCS_CONNECTION_INFO sConnInfo[MAX_CONNECTIONS];
} ConnectionsS;


/*------------------------------------------------------------------------*\
  GLOBAL DATA
\*------------------------------------------------------------------------*/
HANDLE g_hConStdIn, g_hConStdOut;		  // Console Handle
HBCS g_hScanner = NULL;	  // Handle to the Scanner
ConnectionsS g_sConnections;


void ErrorHandler(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Stupid Error Handler Function
\*------------------------------------------------------------------------*/
	const TCHAR *szMsg, 
	HRESULT hRc
)
{
	wprintf(_T("ERROR: '%s' (rc = 0x%x)\n"), szMsg, hRc);
}


HRESULT CALLBACK ConnectionCallbackFunction(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Callback function for Connection Enumeration
\*------------------------------------------------------------------------*/
	void				*aParam,					
	BCS_CONNECTION_INFO *aConnectionInfo
)
{
	ConnectionsS *pConnections = (ConnectionsS *)aParam;
	int nConnections = pConnections->nConnections;

	if (nConnections >= MAX_CONNECTIONS) {
		return S_OK;
	} 

	memcpy ((void *)&pConnections->sConnInfo[nConnections], aConnectionInfo, sizeof(BCS_CONNECTION_INFO));
	pConnections->nConnections++;

	return S_OK;
}


HRESULT CALLBACK EventCallbackFunction(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Callback function for Scanner Events
\*------------------------------------------------------------------------*/
	void			*pvParam,						
	BCS_EVENT_TYPE	eEventType,						
	void			*pEvent
)						
{
	BCS_DATA *pData;

	switch (eEventType)
	{
		case eEventFlicBanner:						// Banner received from the scanner
			wprintf(_T("Banner received from the scanner\n"));
			printf("'%s'\n", (char *)pEvent);
			break;

		case eEventDataReceived:					// Data received from the scanner
			wprintf(_T("Data received from the scanner:\n"));
			pData = (BCS_DATA *)pEvent;
			if (pData->dwSize == sizeof(BCS_DATA)) {
				printf("'%s'\n", pData->pDataReceived);
			}
			break;

		case eEventBarCodeDataReceived:				// Data identified as a bar code (not implemented)
			wprintf(_T("Data identified as a bar code (not implemented)\n"));
			break;

		case eEventCommandPacket:					// Packet that will be sent to the scanner
			wprintf(_T("Packet that will be sent to the scanner\n"));
			break;

		case eEventResponsePacket:					// Packet received from the scanner
			wprintf(_T("Packet received from the scanner\n"));
			break;

		case eEventBluetoothAttemptingToConnect:	// Attempting to connect with Bluetooth device
			wprintf(_T("Attempting to connect with Bluetooth device\n"));
			break;

		case eEventBluetoothConnectionLost:			// Connection with Bluetooth device lost
			wprintf(_T("Connection with Bluetooth device lost\n"));
			break;

		case eEventBluetoothDeviceConnected:		// Bluetooth connection established
			wprintf(_T("Bluetooth connection established\n"));
			break;

		default:
			break;
		}	

	return S_OK;
}		// end of Rov_CallbackEvent


HRESULT CALLBACK BarCodeCallbackFunction(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Callback function for Downloaded Barcodes
\*------------------------------------------------------------------------*/
    void        *pvParam,				// IN  Parameter passed into BcsGetBarCodesA as the pvParam
	const TCHAR *pszSymbology,			// IN  Symbol Identifier (This value is dependent on the setting of the SymIdType property)
	const TCHAR *pszTimestamp,			// IN  Timestamp (This pointer value is only valid if Timestamp = True, otherwize the the pointer value is NULL)
	const TCHAR *pszTimeType,			// IN  Time Type (This pointer value is only valid if Timestamp = True, otherwize the the pointer value is NULL)
										//	   Possible Values: "ok" = Actual Time, or e1, e2, ..., e<N> = Estimated time after N battery events)
	const TCHAR *pszBarCode,			// IN  Bar code data
	int          cchBarCode,			// IN  Length of the bar code data in bytes

	BCS_DOWNLOAD_INFO *pDownloadInfo	// IN  Supplemental information in download packet
)
{
	wprintf(_T("'%s' (TimeStamp = '%s', TimeType = '%s', SymId = '%s'\n"), pszBarCode, pszTimestamp, pszTimeType, pszSymbology);

	return S_OK;
}



HRESULT EnumConnections(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Enumerate Possible Scanner Connections
\*------------------------------------------------------------------------*/
	void
)
{
	HRESULT	hRc;

	memset(&g_sConnections, 0, sizeof(ConnectionsS));

	hRc = BcsEnumConnections(
		NULL,							// IN Reserved, must be NULL.
		NULL,							// IN  Window handle to be used as the parent window for any dialogs displayed by this function
		ConnectionCallbackFunction,		// IN A callback that will be called for each Connection.
										//	If the callback returns >= 0, the enumeration will continue.
										//	If the callback returns < 0, the enumeration will stop.
		(void*) &g_sConnections,		// IN  Parameter to be passed to pfnConnectionCallback as the pvParam
		BCS_ENUM_SERIAL);				// IN  Bitwise or'd combination of BCS_ENUM bits
		
	if (FAILED(hRc))
	{
		ErrorHandler(_T("BcsEnumConnections() failed"), hRc);
		return hRc;
	}

	wprintf(_T("Potential Scanner Connections Found:\n"));
	for (int i = 0; i < g_sConnections.nConnections; i++) {
		wprintf(_T("%d: %s - %s\n"), i, g_sConnections.sConnInfo[i].szPortName, g_sConnections.sConnInfo[i].szName);
	}

	return S_OK;
}


HRESULT OpenConnection(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Open Scanner Connection
\*------------------------------------------------------------------------*/
	int iConnId
)
{
	HRESULT hRc;
	DWORD dwBaudRate;

	if (iConnId >= g_sConnections.nConnections) 
	{
		wprintf(_T("Invalid Connection Id: %d, Max Connection Id: %d\n"), iConnId, g_sConnections.nConnections-1);
		return E_FAIL;
	}

	hRc = BcsOpenConnection(
		&g_hScanner,							// OUT A connection handle for this connection
		&g_sConnections.sConnInfo[iConnId],		// IN  A connection information structure (usually retrieved via BcsEnumConnections)
		BCS_UNKNOWN_BAUDRATE,					// IN  A specific baud rate (4800|9600|19200|38400|57600|115200) or BCS_UNKNOWN_BAUDRATE
		&dwBaudRate,							// OUT Baud rate of the connection.  Can be NULL.
		EventCallbackFunction,					// IN  A callback that will be called for each event.
		NULL,									// IN  Parameter to be passed to pfnEventCallback as the pvParam
		BCS_OPEN_GET_BANNER						// IN  Bitwise or'd combination of BCS_OPEN bits or 0
    );

	if (SUCCEEDED(hRc))
	{
		wprintf(_T("Opened Scanner Connection at %d bauds\n"), dwBaudRate);
	}
	else 
	{
		ErrorHandler(_T("BcsOpenConnection() failed"), hRc);
	}

	return hRc;
}


HRESULT DisplayConnectionProperties(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Display Connection Properties
\*------------------------------------------------------------------------*/
	int iConnId
)
{
	HRESULT hRc;

	if (iConnId >= g_sConnections.nConnections) 
	{
		wprintf(_T("Invalid Connection Id: %d, Max Connection Id: %d\n"), iConnId, g_sConnections.nConnections-1);
		return E_FAIL;
	}

	hRc = BcsDisplayConnectionProperties(
		NULL,						// IN  Window handle to be used as the parent window for dialog displayed by this function
		&g_sConnections.sConnInfo[iConnId],	// IN  A connection information structure (usually retrieved via BcsEnumConnections)
		0									// IN  Reserved, must be 0
    );

	if (FAILED(hRc))
	{
		ErrorHandler(_T("BcsDisplayConnectionProperties() failed"), hRc);
	}

	return hRc;

}

HRESULT CloseConnection(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Close Scanner Connection
\*------------------------------------------------------------------------*/
	void
)
{
	HRESULT hRc;

	hRc = BcsCloseConnection(g_hScanner);
	if (SUCCEEDED(hRc)) {
		wprintf(_T("Scanner Connection Closed\n"));
	} else {
		ErrorHandler(_T("BcsCloseConnection() failed"), hRc);
	}

	return hRc;
}


HRESULT GetBarCodeCount(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Get BarCode Count from the Scanner
\*------------------------------------------------------------------------*/
	void
)
{
	HRESULT hRc;
	int nCount = 0;

	// Get the number of available bar codes
	hRc = BcsGetBarCodeCount(
		g_hScanner, // IN  The connection handle provided by BcsOpenConnection
		&nCount		// OUT Number of bar codes available to be retreived
	);

	if (SUCCEEDED(hRc)) {
		wprintf(_T("Number of BarCodes = %d\n"), nCount);
	} else {
		ErrorHandler(_T("BcsGetBarCodeCount() failed"), hRc);
	}
	return hRc;
}


HRESULT GetBarCodes(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Get BarCodes from the Scanner
\*------------------------------------------------------------------------*/
	void
)
{
	HRESULT hRc;
	int nPacketId;
	int nRemainingBarCodes;

	hRc = BcsGetBarCodes(
		g_hScanner,				// IN  The connection handle provided by BcsOpenConnection
		BCS_BC_NO_PACKET_ID,	// IN  Packet identifier of packet to delete or BCS_BC_NO_PACKET_ID to skip the delete
		BCS_BC_GET_ALL,			// IN  Maximum number of bar codes to retrieve or BCS_BC_GET_ALL to get all available bar codes
		BarCodeCallbackFunction,// IN  A callback that will be called for each Connection.
	    NULL,					// IN  Parameter to be passed to pfnBarCodeCallback as the pvParam
		&nPacketId,				// OUT Pointer to the packet identifier for use in deleting the bar codes
		&nRemainingBarCodes,	// OUT Number of bar codes remaining after download (and delete if BCS_BC_AUTO_DELETE)
		BCS_BC_AUTO_DELETE		// IN  Bitwise or'd combination of BCS_BC_AUTO_DELETE and BCS_BC_ADJUST_TIMESTAMPS or 0
	);

	if (SUCCEEDED(hRc)) {
		wprintf(_T("Downloaded Barcodes, DataId = %d, Barcodes Left = %d\n"), nPacketId, nRemainingBarCodes);
	} else {
		ErrorHandler(_T("BcsGetBarCodes() failed"), hRc);
	}
	return hRc;
}


HRESULT Command(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Issue a XSP command to the scanner
\*------------------------------------------------------------------------*/
	const char *szCmd
)
{
	HRESULT hRc;

	printf("Command: '%s'\n", szCmd);

	hRc = BcsCommandA(	// Send a Command to the Scanner
		g_hScanner,		// IN  The connection handle provided by BcsOpenConnection
		szCmd,			// IN  ASCII version of the command
		LOCALE_SYSTEM_DEFAULT,	// IN  Locale used to both interpret pVarArg and format pVarResult
		NULL,			// IN  Command argument
		NULL,			// OUT Data returned from Command
	    0				// IN  Accepts 0, BCS_XSP_XML or BCS_XSP_BSTR
    );

	if (FAILED(hRc)) {
		ErrorHandler(_T("BcsCommandA() failed"), hRc);
	}
	return hRc;
}


HRESULT GetProp(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Get A Property value from the Scanner
\*------------------------------------------------------------------------*/
	const char *szPropName
)
{
	HRESULT hRc;
	VARIANT varResult;

	hRc = BcsGetPropA(	// Get a Scanner Configuration Property
		g_hScanner,			// IN  The connection handle provided by BcsOpenConnection
		szPropName,			// IN  ASCII version of property name
		LOCALE_SYSTEM_DEFAULT,	// IN  Locale used to format pVarResult
		&varResult,		// OUT Returned propery value
		0			// IN  Accepts 0, BCS_XSP_XML or BCS_XSP_BSTR
	);

	if (SUCCEEDED(hRc)) {
		printf("%s = ", szPropName);
		switch (varResult.vt) {
			case VT_BOOL: 
				wprintf(_T("%d\n"), varResult.boolVal);
				break;
			case VT_BSTR:
				wprintf(_T("%s\n"), varResult.bstrVal);
				break;
			default:
				break;
		}
	} else {
		ErrorHandler(_T("BcsGetPropA() failed"), hRc);
	}

	VariantClear(&varResult);

	return hRc;
}


void CmdHelp(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Command Help
\*------------------------------------------------------------------------*/
	void
)
{
	wprintf(_T("enum - Enumerate possible scanner connections\n"));
	wprintf(_T("open <id> - Open a connection, id = connection identifier listed by enum (0,1,2,...)\n"));
	wprintf(_T("disp <id> - Display connection properties, id = connection identifier listed by enum (0,1,2,...)\n"));
	wprintf(_T("close - Close scanneer connection\n"));
	wprintf(_T("count - Get Number of Barcodes in the scanner\n"));
	wprintf(_T("down - Download Barcodes\n"));
	wprintf(_T("get <propname> - Get Property value\n"));
	wprintf(_T("!<command> - Send an XSP command to the scanner\n"));
	wprintf(_T("help - Display this help\n"));
	wprintf(_T("exit - Exit from the app\n"));
}

int _tmain(
/*------------------------------------------------------------------------*\
  DESCRIPTION
	Main
\*------------------------------------------------------------------------*/
	int argc, 
	_TCHAR* argv[]
)
{
	DWORD dwNumRead;
	//DWORD dwNumWrite; 
	char szCmd[MAX_XSP_COMMAND];
	int  i;

	// Get handles to Console Standard In and Out
	g_hConStdIn = GetStdHandle(STD_INPUT_HANDLE);
	g_hConStdOut = GetStdHandle(STD_OUTPUT_HANDLE);

	// Invalid if equal
	if (g_hConStdIn == g_hConStdOut) {
		ErrorHandler(_T("GetStdHandle() failed"), GetLastError());
	}

	::CoInitialize( NULL );

	SetConsoleTitle(_T("BcsConsole - Simple Windows SDK C Api Sample"));

	wprintf(_T("BcsConsole Running ...\n"));

	do {

		// Read command from console
		//WriteFile(g_hConStdOut, "> ", 2, &dwNumWrite, 0);
		ReadFile(g_hConStdIn, szCmd, MAX_XSP_COMMAND, &dwNumRead, 0);
		for (i = dwNumRead; i < MAX_XSP_COMMAND; i++) {
			szCmd[i] = 0; // Make zero terminated
		} 
		// Remove CRLF
		if (dwNumRead >= 2) {
			szCmd[dwNumRead-1] = 0;
			szCmd[dwNumRead-2] = 0;
		}

		// Write back to Console
		//WriteFile(g_hConStdOut, szCmd, strlen(szCmd), &dwNumWrite, 0);

		if (_strnicmp(szCmd,"enum",4)==0) {
			EnumConnections();
		} 
		else if (_strnicmp(szCmd,"open",4)==0) {
			int iConn = atoi(&szCmd[5]);
			OpenConnection(iConn);
		} 
		else if (_strnicmp(szCmd,"disp",4)==0) {
			int iConn = atoi(&szCmd[5]);
			DisplayConnectionProperties(iConn);
		}
		else if (_strnicmp(szCmd,"close",5)==0) {
			CloseConnection();
		} 
		else if (_strnicmp(szCmd,"count",5)==0) {
			GetBarCodeCount();
		}
		else if (_strnicmp(szCmd,"down",4)==0) {
			GetBarCodes();
		}
		else if (_strnicmp(szCmd,"get",3)==0) {
			GetProp(&szCmd[3]);
		}
		else if (szCmd[0] == '!') {
			Command(&szCmd[1]);
		}
		else if (_strnicmp(szCmd,"help",4)==0) {
			CmdHelp();
		}
		else if (_strnicmp(szCmd,"exit",4)!=0) {
			wprintf(_T("Invalid command, use: help, exit, enum, open <id>, disp <id>, count, down, get <prop>, !<command>\n"));
		}

	} while ( _strnicmp(szCmd, "exit", 4) != 0);

	::CoUninitialize();

	return 0;
}


