// Scanner Class Library - SCL version 0.01
// Copyright (C) Microvision, Inc. All rights reserved.

#ifndef __SCLKEYS_H__
#define __SCLKEYS_H__

#pragma once

#ifndef __cplusplus
  #error SCL requires C++ compilation (use a .cpp suffix)
#endif

#ifndef _WIN32_WCE
#if (WINVER < 0x0500)
  #error sclkeys.h.h requires WINVER >= 0x0500
#endif
#endif

///////////////////////////////////////////////////////////////////////////////
// Classes in this file:
//
// CKeystrokes

const int VK_Base = 0xE000;	// Start of Unicode Private Use Area

// uMapType values for MapVirtualKey
enum {
	eVirtualKeyToScanCode	   = 0,	// uCode is a virtual-key code and is translated into a scan code. If it is a virtual-key code that does not distinguish between left- and right-hand keys, the left-hand scan code is returned. If there is no translation, the function returns 0. 
	eScanCodeToVirtualKey	   = 1,	// uCode is a scan code and is translated into a virtual-key code that does not distinguish between left- and right-hand keys. If there is no translation, the function returns 0. 
	eVirtualKeyToUnshiftedChar = 2,	// uCode is a virtual-key code and is translated into an unshifted character value in the low-order word of the return value. Dead keys (diacritics) are indicated by setting the top bit of the return value. If there is no translation, the function returns 0. 
	eScanCodeToVirtualKeyLR	   = 3	// uCode is a scan code and is translated into a virtual-key code that distinguishes between left- and right-hand keys. If there is no translation, the function returns 0. 
};

/// Synthesizes keystrokes.
/// Callback allows another class to format and send the keystrokes

typedef LRESULT (WINAPI *PFN_SEND_KEYSTROKES_CALLBACK)( void *pvParam, const TCHAR *pszBarCode, int nLength );
//#define VK_PACKET         0xE7
//#define KEYEVENTF_UNICODE     0x0004

class CKeystrokes
{
	PFN_SEND_KEYSTROKES_CALLBACK m_pfnCallback;
	void *m_pvParam;

public:
	CKeystrokes() :
		m_pfnCallback(0),
		m_pvParam(0)
	{}

	LRESULT RegisterCallback( PFN_SEND_KEYSTROKES_CALLBACK pfnCallback, void *pvParam )
	{
		ATLASSERT( m_pfnCallback == NULL );

		m_pfnCallback	= pfnCallback;
		m_pvParam		= pvParam;

		return ERROR_SUCCESS;
	}

	// This method sends a character, either by sythesizing keystrokes or by calling SendInput with the KEYEVENTF_UNICODE flag.

	LRESULT SendChar( wchar_t wc, WORD wShiftState = 0 )
	{
		ATLTRACE( _T("\nSendChar( wc: %x wShiftState: %x )\n"), wc, wShiftState );

#ifndef _WIN32_WCE
		// Check the parameters
		ATLASSERT( !(wShiftState && wc > 255) );				// Ctrl key cannot be applied to characters with value greater then 255
		if (wShiftState && wc > 255)
			return ERROR_INVALID_PARAMETER;

		// Determine whether the character should be sent using VkKeyScanEx based WM_KEYDOWN and WM_KEYUP messages or as Unicode
		// packets (sent via the SendInput function using the KEYEVENTF_UNICODE flag).

		bool bSendAsKeystrokes = wShiftState != 0;				// Send data as a VkKeyScanEx based WM_KEYDOWN and WM_KEYUP messages

		switch (wc) {											// Handle exceptions
		case VK_TAB:
		case VK_RETURN:
		case VK_SPACE:
			bSendAsKeystrokes = true;							// Do not send as a Unicode packet
		}

		// Get the GUI thread of the application for use in determining the keyboard layout
		GUITHREADINFO gti = {sizeof(gti)};						// Get the GUI thread info for the foreground window
		BOOL bHaveThreadInfo = GetGUIThreadInfo( NULL, &gti );
		ATLASSERT( bHaveThreadInfo );							// This should never fail

		// Get the keyboard layout for the target thread
		HKL hkl = GetKeyboardLayout( wShiftState == 0 && bHaveThreadInfo && IsWindow(gti.hwndFocus) ? GetWindowThreadProcessId(gti.hwndFocus, NULL) : 0 );

		BYTE	byVkCode = 0;									//The virtual key code

		if (wc >= 0xE000 && wc <= 0xF8FF)						// Encoded virtual key
			byVkCode = (wc - 0xE000) & 0xff;					// Isolate the virtual key
		else if (bSendAsKeystrokes) {							// Attempt to use VkKeyScanEx based WM_KEYDOWN and WM_KEYUP messages 
			SHORT wVkCode = VkKeyScanExW( wc, hkl );			// Get virtual-key code and shift state for the current keyboard
			ATLTRACE( _T("VkKeyScanExW returned: %x\n"), wVkCode );
			byVkCode	 = LOBYTE( wVkCode );					// Isolate the virtual key
			wShiftState	|= HIBYTE( wVkCode );					// Bitwise or in the shift state bits
		}

		if (byVkCode) {
			// Get the scan codes for the shift state keys that are required for this character
			BYTE byShiftScanCode	= (wShiftState & 1) ? LOBYTE( MapVirtualKeyEx(	VK_SHIFT,		eVirtualKeyToScanCode, hkl) ) : 0;
			BYTE byControlScanCode	= (wShiftState & 2) ? LOBYTE( MapVirtualKeyEx(	VK_CONTROL,		eVirtualKeyToScanCode, hkl) ) : 0;
			BYTE byMenuScanCode		= (wShiftState & 4) ? LOBYTE( MapVirtualKeyEx(	VK_MENU,		eVirtualKeyToScanCode, hkl) ) : 0;

			ATLTRACE( _T("Generating shift state events: %x\n"), wShiftState );

			if (wShiftState & 1)	keybd_event( VK_SHIFT,		byShiftScanCode,	0, 0 );
			if (wShiftState & 2)	keybd_event( VK_CONTROL,	byControlScanCode,	0, 0 );
			if (wShiftState & 4)	keybd_event( VK_MENU,		byMenuScanCode,		0, 0 );

			BYTE byScanCode = LOBYTE( MapVirtualKeyEx(byVkCode, eVirtualKeyToScanCode, hkl) );

			ATLTRACE( _T("Sending byVkCode: %x byScanCode: %x\n"), byVkCode, byScanCode );

			keybd_event( byVkCode, byScanCode, 0, 0 );			// Simulate a key press and release
			keybd_event( byVkCode, byScanCode, KEYEVENTF_KEYUP, 0 );

			if (wShiftState & 1)	keybd_event( VK_SHIFT,		byShiftScanCode,	KEYEVENTF_KEYUP, 0);
			if (wShiftState & 2)	keybd_event( VK_CONTROL,	byControlScanCode,	KEYEVENTF_KEYUP, 0);
			if (wShiftState & 4)	keybd_event( VK_MENU,		byMenuScanCode,		KEYEVENTF_KEYUP, 0);

			return ERROR_SUCCESS;
		}
		else if (wc != 0xE000) {
			INPUT input[2] = {0};

			input[0].type		= INPUT_KEYBOARD;
			input[0].ki.dwFlags	= KEYEVENTF_UNICODE;
			input[0].ki.wScan	= wc;

			input[1].type		= INPUT_KEYBOARD;
			input[1].ki.dwFlags	= KEYEVENTF_UNICODE | KEYEVENTF_KEYUP;
			input[1].ki.wScan	= wc;

			// The SendInput function returns the number of events that it successfully inserted into the keyboard.
			// If the function returns zero, the input was already blocked by another thread.

			ATLTRACE( ("Sending Unicode character: %x\n"), wc );
			return ::SendInput(2, input, sizeof(INPUT)) == 2 ? ERROR_SUCCESS : ::GetLastError();
		}

		return ERROR_INVALID_PARAMETER;
#else
		return ERROR_SUCCESS;
#endif
	}

	LRESULT Send( const TCHAR *pszBarCode, int nLength = -1 )
	{
		ATLASSERT( pszBarCode );

		if (nLength < 0)										// Need to find the length
			nLength = (int)_tcslen( pszBarCode );

		if (m_pfnCallback)										// Call the registered callback
			return (*m_pfnCallback)( m_pvParam, pszBarCode, nLength );

		while (nLength-- > 0 && *pszBarCode)
			SendChar( *pszBarCode++ );

		return ERROR_SUCCESS;
	}
};

#endif // __SCLKEYS_H__
