// PageDlgs.h : interfaces of the page dialog classes
//
/////////////////////////////////////////////////////////////////////////////

#pragma once

#include <atldlgs.h>						// CPropertySheet
#include <atlddx.h>							// CWinDataExchange
#include <atlctrlx.h>						// CWaitCursor

#include "FileVersion.h"

LRESULT CALLBACK SendKeystrokesCallback( void *pvParam, const TCHAR *pszBarCode, int nLength );

static CConnection *g_pConnection;			// Pointer to the current connection object

/// Implements the About property page
/// Displays API version number

class CAboutPage : public CPropertyPageImpl<CAboutPage>
{
    // Maps
    BEGIN_MSG_MAP_EX(CAboutPage)
		MSG_WM_INITDIALOG(		OnInitDialog	)
        CHAIN_MSG_MAP(CPropertyPageImpl<CAboutPage>)
    END_MSG_MAP()

    // Message handlers
	BOOL OnInitDialog( HWND /*hWndFocus*/, LPARAM /*lParam*/ )
	{
		// Center the PropertySheet dialog on the screen
		CWindow wndParent;
		wndParent.Attach( GetParent() );
		if ( !wndParent.IsWindowVisible() )
			wndParent.CenterWindow();

		// Display full version numbers including the build number
		TCHAR szVer[32];

		// Build App version string
		StringCbPrintf( szVer, sizeof szVer, _T("%d.%d.0.%d"), VER_MAJOR, VER_MINOR, VER_BUILD );

		// Display App version number
		::SetWindowText( GetDlgItem(IDC_APPVER), szVer );

		// Display the Bcs SDK DLL file version string
		try {
			CVersion v( _T(BCS_ORIGINAL_FILENAME) );
			::SetWindowText( GetDlgItem(IDC_APIVER), v.FileVersion() );
		}
		catch (const std::exception& e) {
			e;								// Prevent compiler warning
			ATLTRACE( _T("Version info exception: %s\n\n"), e.what() );
		}

		return TRUE;
	}

public:
    enum { IDD = IDD_ABOUT };
};

// Declare the callbacks
HRESULT WINAPI EventCallback( void *pvParam, BCS_EVENT_TYPE eEventType, void *pEvent );

HRESULT WINAPI BarCodeCallback(
	void *		 pvParam,
	const TCHAR *pszSymbology,
	const TCHAR *pszTimeType,
	const TCHAR *pszTimeStamp,
	const TCHAR *pszBarCode,
	int			 cchBarCode,
	BCS_DOWNLOAD_INFO *pDownloadInfo
	);

/// Implements the Connections property page
/// Manages the connections list.

class CConnectionsPage : public CPropertyPageImpl<CConnectionsPage>
{
	// Dialog controls
#ifdef USE_LISTVIEW
	CConnectionListViewCtrl m_ConnectionCtrl;	// List of Scanner SDK connections
#else
	CConnectionListCtrl m_ConnectionCtrl;	// List of Scanner SDK connections
#endif
	CButton				m_btnAdd;			// Add new Bluetooth scanner connections
	CButton				m_btnRemove;		// Remove Bluetooth connection configuration
	CButton				m_btnActivate;		// Activate/Deactivate selected scanner connection
	CButton				m_btnProperties;	// Display connection properties
	CWindow				m_wndStatus;		// Display status

	HMODULE				m_hBcs;				// Module handle to Bcs.dll

public:
    // Support functions
	void DisplayStatus( PCTSTR pszMessageString = NULL )
	{
		// Displays a message in the status area of the dialog.

		if (pszMessageString == NULL)
			pszMessageString = _T("Ready");

		m_wndStatus.SetWindowText( pszMessageString );
	}

	void SetDisplayState()
	{
		// Set the Connect and Delete button states (caption and enabled state).
		// Optionally display status

		bool bHaveConnection = g_pConnection != NULL;
		bool bActive = bHaveConnection && g_pConnection->IsConnected();

		m_ConnectionCtrl.EnableWindow( !bActive );
		m_btnAdd.EnableWindow(		   !bActive );
		m_btnRemove.EnableWindow(	   !bActive && bHaveConnection && !g_pConnection->m_ConnectionInfo.IsSerialPort() );
		m_btnActivate.SetWindowText(	bActive ? _T("De&activate") : _T("&Activate") );
		m_btnActivate.EnableWindow(		bHaveConnection );
		m_btnProperties.EnableWindow(	bHaveConnection );

#if 1
		DisplayStatus( bActive ?  _T("Active") : NULL );
#else
		if (bActive)
			DisplayStatus( _T("Active") );
		else
			DisplayMessage( m_ConnectionCtrl.GetLastResult() );
#endif
	}

	void SetConnection()
	{
		// Set g_pConnection member variable based on the current selection.
		// Update the display state.

		g_pConnection = m_ConnectionCtrl.GetConnection();

		SetDisplayState();
	}

	CConnection *GetConnection()
	{
		return g_pConnection;
	}

	HRESULT Activate()
	{
		// Connect a Connection object to a scanner.
		// Updates the display state.

		ATLASSERT( g_pConnection );

		// Display hourglass and message if connection an take more than a quarter of a second
		bool bNotifyUserOfDelay = g_pConnection->MaxConnectTime() > 250;

		CWaitCursor wait(bNotifyUserOfDelay);
		if (bNotifyUserOfDelay)
			DisplayMessage( BCS_MSG_CONNECTING );

		// Attempt connection to scanner
		HRESULT hr = g_pConnection->Connect( EventCallback );

		// Update the display state
		if (SUCCEEDED(hr)) {
			SetDisplayState();

			//if (hr != S_OK)				// Uncomment to display BCS_MSG_SCANNER_NOT_DETECTED, etc.
			//	DisplayMessage( hr );
		}
		else
			DisplayMessage( hr );

		return hr;
	}

	void DeleteConnection( int nSel )
	{
		// Deletes the current connection control string and associated connection object.
		// Sets the new connection control selection and new current connection object pointer.

		HRESULT hr = m_ConnectionCtrl.DeleteConnection( nSel );
		if (FAILED(hr))
			DisplayMessage( hr );

		SetConnection();
	}

	void DisplayMessage( HRESULT hr )
	{
		// Formats and displays an error message.

		TCHAR *pszMsgBuf = NULL;			// Pointer to message buffer

		// Search for the message:
		// 1.) Search the application's message table
		// 2.) Search the SDK DLL's message table
		// 3.) Search the system message table

		DWORD dwChars = FormatMessage(
			FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_HMODULE | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS
			, NULL, hr, 0, (TCHAR *)&pszMsgBuf, 0, NULL );

		// 2.) Search the SDK DLL's message table

		if (dwChars == 0) {
			if (m_hBcs == NULL)
				m_hBcs = LoadLibraryA( BCS_ORIGINAL_FILENAME );

			dwChars = FormatMessage(
				FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_HMODULE | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS
				, m_hBcs, hr, 0, (TCHAR *)&pszMsgBuf, 0, NULL );
		}

		// 3.) Search the system message table for Win32 errors converted to HRESULTs

		if (dwChars == 0) {
			dwChars = FormatMessage(
				FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, NULL, hr, 0, (TCHAR *)&pszMsgBuf, 0, NULL );
		}

		if (pszMsgBuf) {
			DisplayStatus( pszMsgBuf );		// Display the message retrieved via FormatMessage
			LocalFree( pszMsgBuf );			// Free the memory allocated by FormatMessage
		}
		else {
			if (SUCCEEDED(hr))				// Unknown success code
				DisplayStatus();			// Ready
			else {
				TCHAR szErrorMsg[20];		// Build and dispay an error message using the hex error value
				StringCbPrintf( szErrorMsg, sizeof szErrorMsg, _T("Error: (%8.8x)"), hr );
				DisplayStatus( szErrorMsg );
			}
		}
	}

    // Maps
    BEGIN_MSG_MAP_EX(CConnectionsPage)
		MSG_WM_INITDIALOG(		OnInitDialog	)					// 0x0110
		MSG_WM_VKEYTOITEM(		OnVKeyToItem	)					// 0x002E
		MSG_WM_DEVICECHANGE(	OnDeviceChange	)					// 0x0219
#ifdef USE_LISTVIEW
		NOTIFY_HANDLER_EX(  IDC_CONNECTIONS,	LVN_ITEMCHANGED,	OnItemChanged			)
		NOTIFY_HANDLER_EX(  IDC_CONNECTIONS,	LVN_KEYDOWN,		OnKeyDown				)
#else
		COMMAND_HANDLER_EX( IDC_CONNECTIONS,	LBN_SELCHANGE,		OnSelChangeConnections	)
		COMMAND_HANDLER_EX( IDC_CONNECTIONS,	LBN_DBLCLK,			OnDblClkConnections		)
#endif
		COMMAND_HANDLER_EX( IDC_ADD,			BN_CLICKED,			OnClickedAdd			)
		COMMAND_HANDLER_EX( IDC_REMOVE,			BN_CLICKED,			OnClickedRemove			)
		COMMAND_HANDLER_EX( IDC_ACTIVATE,		BN_CLICKED,			OnClickedActivate		)
		COMMAND_HANDLER_EX( IDC_PROPERTIES,		BN_CLICKED,			OnClickedProperties		)

		MESSAGE_HANDLER_EX(						SW_GETBARCODES,		OnGetBarCodes			)
		MESSAGE_HANDLER_EX(						SW_SENDBARCODE,		OnSendBarCode			)
		MESSAGE_HANDLER_EX(						SW_SENDDATA,		OnSendData				)
		MESSAGE_HANDLER_EX(						SW_DISPLAYPACKET,	OnDisplayPacket			)

        CHAIN_MSG_MAP_MEMBER( m_ConnectionCtrl )

        CHAIN_MSG_MAP( CPropertyPageImpl<CConnectionsPage> )

		REFLECT_NOTIFICATIONS()
    END_MSG_MAP()

    // Message handlers
	BOOL OnInitDialog( HWND /*hWndFocus*/, LPARAM /*lParam*/ )
	{
		// Center the PropertySheet dialog on the screen
		CWindow wndParent;
		wndParent.Attach( GetParent() );
		if ( !wndParent.IsWindowVisible() )
			wndParent.CenterWindow();

		// Subclass the connection control and set up the buttons
		m_ConnectionCtrl.SubclassWindow(GetDlgItem(IDC_CONNECTIONS));

		m_btnAdd		= GetDlgItem( IDC_ADD );		// Add button
		m_btnRemove		= GetDlgItem( IDC_REMOVE );		// Remove button
		m_btnActivate	= GetDlgItem( IDC_ACTIVATE );	// Activate button
		m_btnProperties	= GetDlgItem( IDC_PROPERTIES );	// Properties button
		m_wndStatus		= GetDlgItem( IDC_STATUS );		// Status window

		// Initialize the Connections dialog by poplulating the list box and setting initial
		// state of the buttons and edit controls.

		HRESULT hr = m_ConnectionCtrl.Initialize( /*regHive*/ );
		//ATLASSERT( SUCCEEDED(hr) );

#ifdef USE_LISTVIEW
		m_ConnectionCtrl.SetExtendedListViewStyle( LVS_EX_FULLROWSELECT, LVS_EX_FULLROWSELECT );
#endif

		if ( SUCCEEDED(hr) )
			SetConnection();				// Set the connection and update the display
		else
			SetDisplayState();

		if (szInitialConnectionId[0]) {
			int nSel = m_ConnectionCtrl.FindStringExact( -1, szInitialConnectionId );
			if ( nSel >= 0 ) {				// Connection found in list
				m_ConnectionCtrl.SetCurSel( nSel );
				SetConnection();
				Activate();
			}
		}

		return TRUE;
	}

	LRESULT OnVKeyToItem(UINT vKey, int nCaretPos, HWND /*hWndList*/ ) 
	{
		if (vKey == VK_DELETE)
			DeleteConnection( nCaretPos );

		return -1;	// List box should perform the default action in response to the keystroke
	}

	LRESULT OnDeviceChange( UINT uEvent, DWORD dwData )
	{
		LRESULT lRes = TRUE;

		switch (uEvent)	{
		case DBT_USERDEFINED:
			break;
		default:
			lRes = m_ConnectionCtrl.OnDeviceChange( uEvent, dwData );
		}

		SetConnection();

		return lRes;
	}

#ifdef USE_LISTVIEW
	LRESULT OnKeyDown( NMHDR *pnmh )
	{
		NMLVKEYDOWN *pnkd = (NMLVKEYDOWN *)pnmh;

		if (pnkd->wVKey == VK_DELETE)
			DeleteConnection( m_ConnectionCtrl.GetCurSel() );

		return -1;	// List box should perform the default action in response to the keystroke
	}

	LRESULT OnItemChanged( NMHDR *pnmh )
	{
		// Sets g_pConnection member variable and updates display state.

		SetConnection();
		return 0;
	}
#endif

	void OnSelChangeConnections( UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtrl*/ )
	{
		// Sets g_pConnection member variable and updates display state.

		SetConnection();
	}

	void OnDblClkConnections( UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtrl*/ )
	{
		// Connect or disconnect a connection to a scanner.

		SetConnection();
		OnClickedActivate( 0U, 0, NULL );
	}

	void OnClickedAdd( UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtrl*/ )
	{
		DisplayMessage( m_ConnectionCtrl.Discover() );
	}

	void OnClickedActivate( UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtrl*/ )
	{
		// Connect or disconnect a connection to a scanner.

		ATLASSERT( g_pConnection );
		if (g_pConnection == NULL)
			return;

		// Open the App registry key
		CRegKey rkApp;
		LONG lResCreate = rkApp.Create( regHive.GetWritableHive(), pcszAppRegKey );
		ATLASSERT( lResCreate == ERROR_SUCCESS );

		if ( g_pConnection->IsConnected() ) {	// Disconnecting
			HRESULT hr = g_pConnection->Disconnect();
			if (SUCCEEDED(hr)) {				// Disconnect succeded
				SetDisplayState();				// Update display state
				if (lResCreate == ERROR_SUCCESS)
					rkApp.DeleteValue(_T("ConnectionId"));
			}
			else
				DisplayMessage( hr );				// Disconnect failed, report error
		}
		else {									// Connecting
			// Attempt connection to scanner
			HRESULT hr = Activate();

			// Store the Active connection in the registry
			if (SUCCEEDED(hr) && lResCreate == ERROR_SUCCESS)
				rkApp.SetStringValue( _T("ConnectionId"), g_pConnection->m_ConnectionInfo.Name() );
		}
	}

	void OnClickedProperties( UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtrl*/ )
	{
		// Delete a registry entry and the associated list box item.
		g_pConnection->DisplayProperties( m_hWnd );
	}

	void OnClickedRemove( UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtrl*/ )
	{
		// Delete a registry entry and the associated list box item.

		DeleteConnection( m_ConnectionCtrl.GetCurSel() );
	}

public:
    enum { IDD = IDD_CONNECTIONS };

    // Construction
    CConnectionsPage()
	{
		g_pConnection = NULL;				// No current connection object

		m_hBcs = NULL;						// Module handle to Bcs.dll
	}

    ~CConnectionsPage()
	{
		if (m_hBcs)
			FreeLibrary( m_hBcs );			// Module handle to Bcs.dll
	}

	LRESULT OnGetBarCodes(UINT uMsg, WPARAM /*wParam*/, LPARAM /*lParam*/)
	{
		MSG msg;
		if (::PeekMessage( &msg, GetParent(), 0, 0, PM_NOREMOVE | PM_QS_INPUT | PM_QS_PAINT | PM_QS_SENDMESSAGE )) {
			// Should set a timer here so that we delay processing of this message until all other messages are serviced.
			// NOTE:  In an environment that supports OnIdle processing, this function should execute when the input is idle.
			// return 0;
		}

		// Get a chunk of bar codes

		const int nMaxBarCodesPerPacket = g_pConnection->GetBaudRate()/1200 + 6;

		int  nRemainingBarCodes;		// OUT Number of bar codes remaining after download (and delete if BCS_BC_AUTO_DELETE)

		// Get the first chuck of bar codes
		HRESULT hr = BcsGetBarCodes(
			g_pConnection->m_hBcs,		// IN  The connection handle provided by BcsOpenConnection
			BCS_BC_NO_PACKET_ID,		// IN  Packet identifier of packet to delete or BCS_BC_NO_PACKET_ID to skip the delete
			nMaxBarCodesPerPacket,		// IN  Maximum number of bar codes to retrieve or BCS_GET_ALL to get all available bar codes
			BarCodeCallback,			// IN A callback that will be called for each Connection.
										//	If the callback returns >= 0, the enumeration will continue.
										//	If the callback returns < 0, the enumeration will stop.
			this,						// IN  Parameter to be passed to pfnBarCodeCallback as the pvParam
			NULL,						// OUT Pointer to the packet identifier for use in deleting the bar codes
			&nRemainingBarCodes,		// OUT Number of bar codes remaining after download (and delete if BCS_BC_AUTO_DELETE)
			BCS_BC_AUTO_DELETE | BCS_BC_ADJUST_TIMESTAMPS			// IN  0 or BCS_BC_AUTO_DELETE
			);

		// Post a message to ourselves to process the rest of the available bar codes
		if (SUCCEEDED(hr) && nRemainingBarCodes > 0)
			PostMessage( SW_GETBARCODES );

		return 0;
	}

	LRESULT OnSendBarCode(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam)
	{
		// Send the bar code to the trace window or another application

		TCHAR *pszBarCode = (TCHAR *)wParam;
		if (SendBarCode(pszBarCode, lParam)) {
			// Send succeeded, so free the bar code memory
			BOOL bRet = HeapFree( hDataHeap, 0, pszBarCode );
			if (!bRet) {
				DWORD dwErr = ::GetLastError();
				ATLASSERT( dwErr == ERROR_SUCCESS );
			}
		}
		else								// Failed to send bar code, try again later
			PostMessage( SW_SENDBARCODE, wParam, lParam );

		return 0;
	}

	LRESULT OnSendData(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam)
	{
		// Send the bar code to the trace window or another application

		TCHAR *pszData = (TCHAR *)wParam;
		if (SendData(pszData, lParam)) {
			// Send succeeded, so free the bar code memory
			BOOL bRet = HeapFree( hDataHeap, 0, pszData );
			if (!bRet) {
				DWORD dwErr = ::GetLastError();
				ATLASSERT( dwErr == ERROR_SUCCESS );
			}
		}
		else								// Failed to send data, try again later
			PostMessage( SW_SENDDATA, wParam, lParam );

		return 0;
	}

	LRESULT OnDisplayPacket(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam)
	{
		// Send the bar code to the trace window or another application

		TCHAR *pszData = (TCHAR *)wParam;
		DisplayPacket( pszData, lParam );

		// Send succeeded, so free the bar code memory
		BOOL bRet = HeapFree( hDataHeap, 0, pszData );
		if (!bRet) {
			DWORD dwErr = ::GetLastError();
			ATLASSERT( dwErr == ERROR_SUCCESS );
		}

		return 0;
	}
};

HRESULT WINAPI EventCallback( void *pvParam, BCS_EVENT_TYPE eEventType, void *pEvent )
{
	// Process the connection event

	switch (eEventType) {
	case eEventDataReceived: {						// Data received from the scanner
		// Get a BCS_DATA pointer
		BCS_DATA *pData = reinterpret_cast<BCS_DATA *>( pEvent );
		ATLTRACE( "EventCallback - eDataReceived: [%d]%s\n", pData->cbDataReceived, pData->pDataReceived );

		// Search the data buffer for a bar code
		// In order to identify a bar code, rely on an STX prefix and a standard suffix (CR, CRLF, or ETX)

		// Need at least three characters (STX<bar code data><suffix>)
		if (pData->cbDataReceived < 3)
			return BCS_DATA_MORE;

		int nBarCodeStart = 0;				// Start of bar code data

		// Increment past initial newline (may be left over from previous bar code suffix)
		if (pData->pDataReceived[nBarCodeStart] == '\n')
			nBarCodeStart++;

		// If bar code does not start with STX, we can not recognize it as a bar code
		if (pData->pDataReceived[nBarCodeStart] != 2/*STX*/)
			return BCS_DATA_DISCARD;

		nBarCodeStart++;					// Bar code data starts immediately after the STX

		// Search for the first suffix character (CR or ETX)
		for (int i = nBarCodeStart; i < pData->cbDataReceived && i < BCS_MAX_BARCODE_SIZE + 2; i++) {
			if (pData->pDataReceived[i] == '\r' || pData->pDataReceived[i] == 3/*ETX*/) {

				// Found bar code, set the cbDataReceived buffer
				pData->cbDataReceived = i + 1;

				// Check for newline following carriage return
				if( pData->pDataReceived[i] == '\r' && pData->pDataReceived[i+1] == '\n')
					pData->cbDataReceived++;

				// Get a pointer to the connection
				CConnection *pConnection = reinterpret_cast<CConnection *>( pvParam );

				if (bM2M) {
					::PostMessage( pConnection->m_hWndEvents, SW_GETBARCODES, 0, 0 );
					return BCS_DATA_DISCARD;
				}
				else {
					// Get the bar code
					bool bIsSerialConnection = pConnection->m_ConnectionInfo.IsSerialPort();

					// Determine the number of bytes of bar code data
					int cbBarCode = i - nBarCodeStart;
					if (!bIsSerialConnection)
						cbBarCode -= 4;			// Do not include sequence number

					// Allocate storage for the bar code
					const int cbAlloc = (cbBarCode + 1) * sizeof TCHAR;
					TCHAR *pszBarCode = (TCHAR *)HeapAlloc( hDataHeap, 0, cbAlloc );
					if (pszBarCode == NULL)		// This should never happen
						return BCS_DATA_DISCARD;

#ifdef _UNICODE
					int cchBarCode = ::MultiByteToWideChar( CP_ACP, 0, (LPCSTR)&pData->pDataReceived[nBarCodeStart], cbBarCode, pszBarCode, cbAlloc );
#else
					memcpy( pszBarCode, &pData->pDataReceived[nBarCodeStart], cbBarCode );
					int cchBarCode = cbBarCode;
#endif
					pszBarCode[cchBarCode] = '\0';
					ATLTRACE( _T("pszBarCode: %s\n"), pszBarCode );

					// Send data to capture window or send keystrokes
					::PostMessage( pConnection->m_hWndEvents, SW_SENDBARCODE, (WPARAM)pszBarCode, cchBarCode );

					// Send the response
					if (bIsSerialConnection) {
						pData->rgResponse[0] = 6;	//ACK;
						pData->cbResponse = 1;
					}
					else {
						memcpy( pData->rgResponse, &pData->pDataReceived[nBarCodeStart + cbBarCode], 4 );
						pData->rgResponse[4] = 6;	//ACK;
						pData->cbResponse = 5;
					}

					pData->rgResponse[pData->cbResponse] = '\0';
					ATLTRACE( "Response: [%d]%s\n", pData->cbResponse, pData->rgResponse );

					return BCS_DATA_REPLY;
				}
			}
		}

		return BCS_DATA_MORE;				// Failed to find CR or ETX

	}
	case eEventFlicBanner:							// Banner received from the scanner
//	case eEventBarCodeDataReceived:					// Data identified as a bar code (not implemented)
	case eEventCommandPacket:						// Packet that will be sent to the scanner
	case eEventResponsePacket:						// Packet received from the scanner
	case eEventBluetoothAttemptingToConnect:		// Attempting to connect with Bluetooth device
	case eEventBluetoothConnectionLost:				// Connection with Bluetooth device lost
	case eEventBluetoothDeviceConnected:			// Bluetooth connection established
	{
		// Allocate storage for the bar code
		const char *pPacket = reinterpret_cast<const char *>( pEvent );
		int cbPacket = strlen( pPacket );
		const int cbAlloc = (cbPacket + 1) * sizeof TCHAR;
		TCHAR *pszPacket = (TCHAR *)HeapAlloc( hDataHeap, 0, cbAlloc );
		if (pszPacket == NULL)		// This should never happen
			return E_OUTOFMEMORY;

#ifdef _UNICODE
		int cchPacket = ::MultiByteToWideChar( CP_ACP, 0, pPacket, cbPacket, pszPacket, cbAlloc );
#else
		HRESULT hr = StringCbCopy( pszPacket, cbAlloc, pPacket );
		ATLASSERT( SUCCEEDED(hr) );
		int cchPacket = cbPacket;
#endif
		pszPacket[cchPacket] = '\0';
		ATLTRACE( _T("pszPacket: %s\n"), pszPacket );

		// Get a pointer to the connection
		CConnection *pConnection = reinterpret_cast<CConnection *>( pvParam );

		// Send data to capture window or send keystrokes
		::SendMessage( pConnection->m_hWndEvents, SW_DISPLAYPACKET, (WPARAM)pszPacket, cchPacket );
		break;
	}

	default:
		break;
	}

	return BCS_DATA_DISCARD;
}

HRESULT WINAPI BarCodeCallback(
	void *pvParam,
	const TCHAR *pszSymbology,
	const TCHAR *pszTimeStamp,
	const TCHAR *pszTimeType,
	const TCHAR *pszBarCode,
	int cchBarCode,
	BCS_DOWNLOAD_INFO *pDownloadInfo		// IN  Supplemental information in download packet
	)
{
	// Send data to capture window or send keystrokes
	CConnectionsPage *pConnectionsPage = reinterpret_cast<CConnectionsPage *>( pvParam );

	if (bData) {
		TCHAR szData[128];
		PTSTR pszDestEnd;
		HRESULT hr = StringCbPrintfEx( szData, sizeof szData, &pszDestEnd, NULL, 0, _T("%s\t%s\t%s\t%s\r"), pszSymbology, pszTimeStamp, pszTimeType, pszBarCode );
		if (SUCCEEDED(hr))
			SendData( szData, pszDestEnd - szData );

		return hr;
	}

	SendBarCode( pszBarCode, cchBarCode );

	return S_OK;
}

const TCHAR *pcszRegKeyTraceFormat	= _T("TraceFormat");

class CTracePage : public CPropertyPageImpl<CTracePage>, public CWinDataExchange<CTracePage>
{
	// Dialog controls
	CTraceCtrl	m_ctlTrace;
	CButton		m_btnScanner;

	// Member variables
	BOOL		m_bScanner;					// Display unformatted scanner data

	// Support functions
	LRESULT SaveSettings()					// Save the barcode trace settings
	{
		// Open the App registry key
		CRegKey rk;
		LRESULT lRes = rk.Create( regHive.GetWritableHive(), pcszAppRegKey );
		ATLASSERT( lRes == ERROR_SUCCESS );
		if (lRes != ERROR_SUCCESS)
			return lRes;

		// Save the settings
		lRes = rk.SetDWORDValue( pcszRegKeyTraceFormat, m_bScanner );

		if (m_hWnd && lRes == ERROR_SUCCESS)
			SetModified( false );

		return lRes;
	}

	LRESULT LoadSettings()
	{
		// Open the App registry key
		CRegKey rk;
		LRESULT lRes = rk.Open( regHive, pcszAppRegKey, KEY_READ );
		if (lRes != ERROR_SUCCESS) {
			ATLASSERT( lRes == ERROR_FILE_NOT_FOUND );
			return lRes;
		}

		// Load the settings
		DWORD dwValue = 0;

		// Override the default settings if saved values are available
		lRes = rk.QueryDWORDValue( pcszRegKeyTraceFormat, dwValue );
		//REG ATLASSERT( lRes == ERROR_SUCCESS );

		if (lRes == ERROR_SUCCESS)
			m_bScanner = dwValue;

		if (m_hWnd)
			SetModified( false );

		return lRes;
	}

    // Maps
    BEGIN_MSG_MAP_EX(CTracePage)
		MSG_WM_INITDIALOG(		OnInitDialog	)
		COMMAND_RANGE_CODE_HANDLER_EX( IDC_TRACE_SCANNER,	IDC_TRACE_WEDGE,		BN_CLICKED,		OnClickedScanner	)
		COMMAND_HANDLER_EX( ID_EDIT_COPY,			BN_CLICKED,		OnClickedCopy		)
		COMMAND_HANDLER_EX( ID_EDIT_CLEAR_ALL,		BN_CLICKED,		OnClickedClearAll	)
        CHAIN_MSG_MAP(CPropertyPageImpl<CTracePage>)
    END_MSG_MAP()

    BEGIN_DDX_MAP(CTracePage)
    END_DDX_MAP();

    // Message handlers
	BOOL OnInitDialog( HWND /*hWndFocus*/, LPARAM /*lParam*/ )
	{
		m_ctlTrace.SubclassWindow( GetDlgItem(IDC_TRACE_RICHEDIT) );

		m_btnScanner = GetDlgItem( IDC_TRACE_SCANNER );

		if (m_bScanner)
			m_btnScanner.SetCheck( true );
		else {
			CButton m_btnWedge( GetDlgItem(IDC_TRACE_WEDGE) );
			m_btnWedge.SetCheck( true );
		}

		return FALSE;
	}

	void OnClickedScanner( UINT /*uNotifyCode*/, int /*nID*/, HWND /*hWndCtl*/ )
	{
		m_bScanner = m_btnScanner.GetCheck();
		m_ctlTrace.SetFocus();
		SetModified();
	}

	void OnClickedCopy( UINT /*uNotifyCode*/, int /*nID*/, HWND /*hWndCtl*/ )
	{
		CHARRANGE cr;
		m_ctlTrace.GetSel(cr );
		if ( cr.cpMin == cr.cpMax )			// Selection is an insertion point, select all instead
			m_ctlTrace.SetSelAll();

		m_ctlTrace.Copy();					// Copy the selection to the clipboard

		if ( cr.cpMin == cr.cpMax )			// Reset the insertion point
			m_ctlTrace.SetSel( cr );

		m_ctlTrace.SetFocus();
	}

	void OnClickedClearAll( UINT /*uNotifyCode*/, int /*nID*/, HWND /*hWndCtl*/ )
	{
		// Get the default character format
		CHARFORMAT cfDefault = {sizeof(CHARFORMAT)};
		DWORD dwRet = m_ctlTrace.GetDefaultCharFormat( cfDefault );
		cfDefault.dwMask = CFM_COLOR;		// We are only changing the text color

		m_ctlTrace.SetSelAll();				// Select all
		m_ctlTrace.SetSelectionCharFormat( cfDefault );
		m_ctlTrace.Clear();					// Clear all

		m_ctlTrace.SetFocus();
	}

public:
    enum { IDD = IDD_TRACE };

	// Constructors
	CTracePage()
	{
		m_bScanner = true;					// Use raw scanner data

		LoadSettings();						// Ignore errors
	}

	// Overrides
	int OnApply()	{ return SaveSettings() == ERROR_SUCCESS ? PSNRET_NOERROR : PSNRET_INVALID; }
	void OnReset()	{ LoadSettings(); }

	bool SendBarCode( bool bIsWedgeForegroundWindow, const TCHAR *pszBarCode, int cbBarCode )
	{
		// Send data to capture window or send keystrokes
		if ( bIsWedgeForegroundWindow ) {
			if (m_bScanner) {
				if (m_ctlTrace.m_hWnd == NULL)
					return false;

				m_ctlTrace.Trace( pszBarCode, cbBarCode );
				m_ctlTrace.UpdateWindow();
			}
			else
				g_Keystrokes.Send( pszBarCode, cbBarCode );
		}
		else
			g_Keystrokes.Send( pszBarCode, cbBarCode );

		return true;
	}

	bool SendData( bool bIsWedgeForegroundWindow, const TCHAR *pszData, int cbData )
	{
		// Send data to capture window or send keystrokes
		if ( bIsWedgeForegroundWindow ) {
			if (m_bScanner) {
				if (m_ctlTrace.m_hWnd == NULL)
					return false;

				m_ctlTrace.ReplaceSel( pszData, false );
				m_ctlTrace.UpdateWindow();
			}
			else {
				while (cbData-- > 0) {
					if (*pszData != '\n')
						g_Keystrokes.SendChar( *pszData++ );
				}
			}
		}
		else {
			while (cbData-- > 0) {
				if (*pszData != '\n')
					g_Keystrokes.SendChar( *pszData++ );
			}
		}

		return true;
	}
 
	bool DisplayPacket( CWindow &wndParent, const TCHAR *pszData, int cbData )
	{
		// Send data to capture window
		if ( m_bScanner ) {
#if 0
			if (m_ctlTrace.m_hWnd == NULL) {
				wndParent.SendMessage( SW_SETACTIVEPAGE, IDD_TRACE );
				UpdateWindow();
			}
#endif
			if (m_ctlTrace.m_hWnd == NULL)
				return false;

			// Get the default character format
			CHARFORMAT cfDefault = {sizeof(CHARFORMAT)};
			DWORD dwRet = m_ctlTrace.GetDefaultCharFormat( cfDefault );
			cfDefault.dwMask = CFM_COLOR;		// We are only changing the text color

			// Create a gray version of the default char format
			CHARFORMAT cfGrayText = cfDefault;
			cfGrayText.dwEffects = 0;
			cfGrayText.crTextColor = ::GetSysColor( COLOR_GRAYTEXT );

			// Display the data in gray text
			m_ctlTrace.SetSelectionCharFormat( cfGrayText );
			m_ctlTrace.ReplaceSel( pszData, false );
			m_ctlTrace.SetSelectionCharFormat( cfDefault );

			m_ctlTrace.UpdateWindow();
		}

		return true;
	}
 };

/// Implements the Format property page
/// User interface for Keystrokes class.

class CFormatPage : public CPropertyPageImpl<CFormatPage>
{
	// Dialog controls
	CTrimCtrl	m_trimLeft;			// Specifies the number of chars to trim off of the left side of a bar code
	CTrimCtrl	m_trimRight;		// Specifies the number of chars to trim off of the right side of a bar code

	CKeyCtrl	m_keyPrepend;		// Specifies the first key to prepend to the start of a bar code
	CKeyCtrl	m_keyAppend1;		// Specifies the first key to append to the end of a bar code
	CKeyCtrl	m_keyAppend2;		// Specifies the second key to append to the end of a bar code

	// Support functions
	LRESULT SaveSettings()					// Save the barcode format settings
	{
		// Open the App registry key
		CRegKey rk;
		LRESULT lRes = rk.Create( regHive.GetWritableHive(), pcszAppRegKey );
		ATLASSERT( lRes == ERROR_SUCCESS );
		if (lRes != ERROR_SUCCESS)
			return lRes;

		// Save the settings
		m_trimLeft.WriteToRegistry(   rk, _T("TrimLeft")  );
		m_trimRight.WriteToRegistry(  rk, _T("TrimRight") );

		m_keyPrepend.WriteToRegistry( rk, _T("Prepend") );
		m_keyAppend1.WriteToRegistry( rk, _T("Append1") );
		m_keyAppend2.WriteToRegistry( rk, _T("Append2") );

		if (m_hWnd)
			SetModified( false );

		return lRes;
	}

	LRESULT LoadSettings()
	{
		// Open the App registry key
		CRegKey rk;
		LRESULT lRes = rk.Open( regHive, pcszAppRegKey, KEY_READ );
		if (lRes != ERROR_SUCCESS) {
			ATLASSERT( lRes == ERROR_FILE_NOT_FOUND );
			return lRes;
		}

		// Load the settings
		m_trimLeft.ReadFromRegistry(   rk, _T("TrimLeft")  );
		m_trimRight.ReadFromRegistry(  rk, _T("TrimRight") );

		m_keyPrepend.ReadFromRegistry( rk, _T("Prepend") );
		m_keyAppend1.ReadFromRegistry( rk, _T("Append1") );
		m_keyAppend2.ReadFromRegistry( rk, _T("Append2") );

		if (m_hWnd)
			SetModified( false );

		return lRes;
	}

    // Maps
    BEGIN_MSG_MAP_EX(CFormatPage)
		MSG_WM_INITDIALOG(	OnInitDialog	)
		MSG_WM_COMMAND(		OnCommand		)
        CHAIN_MSG_MAP(CPropertyPageImpl<CFormatPage>)
#ifndef _UNICODE
		REFLECT_NOTIFICATIONS()
#endif
    END_MSG_MAP()
 
    // Message handlers
	BOOL OnInitDialog( HWND /*hWndFocus*/, LPARAM /*lParam*/ )
	{
		// Subclass the edit boxes

		m_trimLeft.SubclassWindow(	GetDlgItem(IDC_TRIM_LEFT ) );
		//m_trimRight.SubclassWindow(	GetDlgItem(IDC_TRIM_RIGHT) );

		// Initialize the Trim controls including their buddies

		m_trimLeft.Initialize(  GetDlgItem(IDC_SPIN_TRIM_LEFT ) );
		//m_trimRight.Initialize( GetDlgItem(IDC_SPIN_TRIM_RIGHT) );

		// Subclass the combo boxes

		m_keyPrepend.SubclassWindow( GetDlgItem(IDC_PREPEND) );
		m_keyAppend1.SubclassWindow( GetDlgItem(IDC_APPEND1) );
		m_keyAppend2.SubclassWindow( GetDlgItem(IDC_APPEND2) );

		// Fill in the combo box values

		m_keyPrepend.Initialize( GetDlgItem(IDC_PREFIX_CTRL ) );
		m_keyAppend1.Initialize( GetDlgItem(IDC_SUFFIX1_CTRL) );
		m_keyAppend2.Initialize( GetDlgItem(IDC_SUFFIX2_CTRL) );

		return TRUE;
	}

	void OnCommand( UINT uNotifyCode, int /*nID*/, HWND /*hWndCtrl*/ )
	{
		switch (uNotifyCode) {
		case EN_CHANGE:
		case CBN_SELCHANGE:
		case CBN_EDITCHANGE:
		case BN_CLICKED:
			SetModified();
		}
		SetMsgHandled( false );
	}

public:
    enum { IDD = IDD_FORMAT};

	// Constructors
	CFormatPage()
	{
		g_Keystrokes.RegisterCallback( ::SendKeystrokesCallback, this );

		m_keyAppend1.SetChar( VK_Base + VK_RETURN );

		LoadSettings();						// Ignore errors
	}

	// Overrides
	int OnApply()	{ return SaveSettings() == ERROR_SUCCESS ? PSNRET_NOERROR : PSNRET_INVALID; }
	void OnReset()	{ LoadSettings(); }

	// Callbacks
	LRESULT CALLBACK SendKeystrokesCallback( const TCHAR *pszBarCode, int nLength )
	{
		g_Keystrokes.SendChar( m_keyPrepend.GetChar(), m_keyPrepend.GetState() );

		int nCharsTrimLeft = m_trimLeft.GetValue();

		if (nLength > nCharsTrimLeft) {						// Send the bar code characters
			pszBarCode += nCharsTrimLeft;
			nLength -= nCharsTrimLeft;
			nLength -= m_trimRight.GetValue();

			while (nLength-- > 0)
				g_Keystrokes.SendChar( *pszBarCode++ );
		}

		g_Keystrokes.SendChar( m_keyAppend1.GetChar(), m_keyAppend1.GetState() );
		g_Keystrokes.SendChar( m_keyAppend2.GetChar(), m_keyAppend2.GetState() );

		return ERROR_SUCCESS;
	}
};

LRESULT CALLBACK SendKeystrokesCallback( void *pvParam, const TCHAR *pszBarCode, int nLength )
{
	CFormatPage* pFormatPage = reinterpret_cast<CFormatPage *>( pvParam );

	return pFormatPage->SendKeystrokesCallback( pszBarCode, nLength );
}

static const int nSymbologies = 24;	//22;
static const TCHAR *pszSymbologies[nSymbologies] = {
	_T("DecodeUPCA_EAN13"),
	_T("DecodeUPCE"),
	_T("DecodeEAN8"),
	_T("DecodeC128"),
	_T("DecodeC39"),
	_T("DecodeITF"),
	_T("DecodeRSSL"),
	_T("DecodeRSSS"),
	_T("DecodeRSS14"),
	_T("DecodeGS1_128"),
	_T("DecodeCodabar"),
	_T("VerifyC39CheckSum"),
	_T("StripC39CheckSum"),
	_T("StripUPCCheckSum"),
	_T("ExpandUPCEToUPCA"),
	_T("ExpandUPCAToEAN13"),
	_T("ExpandUPCEANTo14"),
	_T("VerifyITFCheckSum"),
	_T("StripITFCheckSum"),
	_T("DecodeSupps"),
	_T("RequireSupps"),
	_T("CombineSupps"),
	_T("BcDataAck"),
	_T("PerfData")
};

class CSettingsPage : public CPropertyPageImpl<CSettingsPage>
{
	// Dialog controls
	CComboBox	m_cbSymProp;				// Display and set symbology decoding
	CComboBox	m_cbSymPropVal;				// Symbology values IDC_SYMID
	CComboBox	m_cbSymIdTypeVal;			// Symbology identifier values
	CButton		m_chkTimestamp;			// Timestamps enabled: checked - true, unchecked - false

	// Member variables

    // Support functions

    // Maps
    BEGIN_MSG_MAP_EX(CSettingsPage)
		MSG_WM_INITDIALOG( OnInitDialog )
		COMMAND_HANDLER_EX(	IDC_SYM_PROP,		CBN_SELCHANGE,	OnSelChangeSymProp		)
		COMMAND_HANDLER_EX(	IDC_SYM_PROP_VAL,	CBN_SELCHANGE,	OnSelChangeSymPropVal	)
		COMMAND_HANDLER_EX(	IDC_SYM_ID_TYPE,	CBN_SELCHANGE,	OnSelChangeSymIdTypeVal	)
		COMMAND_HANDLER_EX(	IDC_TIMESTAMP,		BN_CLICKED,		OnClickedTimestamp		)
        COMMAND_HANDLER_EX( IDC_SET_TIME,		BN_CLICKED,		OnClickedSetTime		)
CHAIN_MSG_MAP(CPropertyPageImpl<CSettingsPage>)
    END_MSG_MAP()

    // Message handlers
	BOOL OnInitDialog( HWND /*hWndFocus*/, LPARAM /*lParam*/ )
	{
		// Initialize dialog conrol objects
		m_cbSymProp			= GetDlgItem( IDC_SYM_PROP );		// Display and set symbology decoding
		m_cbSymPropVal		= GetDlgItem( IDC_SYM_PROP_VAL );	// Symbology values
		m_cbSymIdTypeVal	= GetDlgItem( IDC_SYM_ID_TYPE );	// Symbology identifier type values
		m_chkTimestamp		= GetDlgItem( IDC_TIMESTAMP );		// Hide the dialog on application start up

		// Initialize dialog to match member variables
		for (int i = 0; i < nSymbologies; i++)
			m_cbSymProp.AddString( pszSymbologies[i] );
		m_cbSymProp.SetCurSel( 0 );

		m_cbSymPropVal.AddString( _T("True") );
		m_cbSymPropVal.AddString( _T("False") );

		m_cbSymIdTypeVal.AddString( _T("Off") );
		m_cbSymIdTypeVal.AddString( _T("AIM") );
		m_cbSymIdTypeVal.AddString( _T("Custom") );

		return TRUE;
	}

	void OnSelChangeSymProp( UINT uNotifyCode, int /*nID*/, HWND /*hWndCtrl*/ )	
	{
		CWaitCursor wait;
		int nSymPropSel = m_cbSymProp.GetCurSel();
		if (nSymPropSel >= 0) {
			TCHAR szSymProp[32];
			int nChars = m_cbSymProp.GetLBText( nSymPropSel, szSymProp );
			ATLASSERT( nChars < sizeof(szSymProp)/sizeof(TCHAR) );
			CComVariant varSymProp;
			HRESULT hr = g_pConnection->GetProp( szSymProp, &varSymProp );
			if (SUCCEEDED(hr) && varSymProp.vt == VT_BOOL )
				m_cbSymPropVal.SetCurSel( varSymProp.boolVal ? 0 : 1 );
			else
				m_cbSymPropVal.SetCurSel( -1 );
		}
	}

	void OnSelChangeSymPropVal( UINT uNotifyCode, int /*nID*/, HWND /*hWndCtrl*/ )
	{
		CWaitCursor wait;
		int nSymPropValSel = m_cbSymPropVal.GetCurSel();
		if (nSymPropValSel >= 0) {
			int nSymPropSel = m_cbSymProp.GetCurSel();
			if (nSymPropSel >= 0) {
				TCHAR szSymProp[32];
				int nChars = m_cbSymProp.GetLBText( nSymPropSel, szSymProp );
				ATLASSERT( nChars < sizeof(szSymProp)/sizeof(TCHAR) );
				CComVariant varSymProp( !nSymPropValSel );
				HRESULT hr = g_pConnection->SetProp( szSymProp, &varSymProp );
				if (FAILED(hr)) {
					// Should revert back
					m_cbSymPropVal.SetCurSel( -1 );
				}
			}
		}
	}

	void OnSelChangeSymIdTypeVal( UINT uNotifyCode, int /*nID*/, HWND /*hWndCtrl*/ ) 
	{
		CWaitCursor wait;
		int nSymIdTypeValSel = m_cbSymIdTypeVal.GetCurSel();
		if (nSymIdTypeValSel >= 0) {
			TCHAR szSymIdTypeVal[32];
			int nChars = m_cbSymIdTypeVal.GetLBText( nSymIdTypeValSel, szSymIdTypeVal );
			ATLASSERT( nChars < sizeof(szSymIdTypeVal)/sizeof(TCHAR) );
			CComVariant varSymIdTypeVal = szSymIdTypeVal;
			HRESULT hr = g_pConnection->SetProp( _T("SymIdType"), &varSymIdTypeVal );
			if (FAILED(hr)) {
				// Should revert back
				m_cbSymIdTypeVal.SetCurSel( -1 );
			}
		}
	}

	void OnClickedTimestamp( UINT uNotifyCode, int /*nID*/, HWND /*hWndCtrl*/ )
	{
		CWaitCursor wait;
		CComVariant varTimestamp = m_chkTimestamp.GetCheck() == BST_CHECKED;
		HRESULT hr = g_pConnection->SetProp( _T("Timestamp"), &varTimestamp );
		if (FAILED(hr)) {
			// Attempt to revert back
			hr = g_pConnection->GetProp( _T("Timestamp"), &varTimestamp );
			if (SUCCEEDED(hr) && varTimestamp.vt == VT_BOOL )
				m_chkTimestamp.SetCheck( varTimestamp.boolVal );
			else
				m_chkTimestamp.SetCheck( !m_chkTimestamp.GetCheck() );	// Need 3 State, set to BST_INDETERMINATE!
		}
	}
//Set time, yyyy-mm-dd hh:mm:ss 

	void OnClickedSetTime( UINT uNotifyCode, int /*nID*/, HWND /*hWndCtrl*/ )
	{
		CWaitCursor wait;
		UDATE udate;
		GetLocalTime( &udate.st );			// Use the WIN32 API to get the system time

		VARIANT varTime = {VT_DATE};
		VarDateFromUdate( &udate, 0, &varTime.date );
		HRESULT hr = g_pConnection->SetProp( _T("Time"), &varTime );
		ATLASSERT( SUCCEEDED(hr) );
	}

public:
    enum { IDD = IDD_SETTINGS };
 
	int OnSetActive()
	{
		// Clear out any previous values
		m_cbSymPropVal.SetCurSel( -1 );
		m_cbSymIdTypeVal.SetCurSel( -1 );
		m_chkTimestamp.SetCheck( /*BST_INDETERMINATE*/BST_UNCHECKED );

		// Determine whether we have an active connection
		bool bActive = g_pConnection && g_pConnection->IsConnected();

		// Enable or disable the controls accordingly
		m_cbSymProp.EnableWindow( bActive );
		m_cbSymPropVal.EnableWindow( bActive );
		m_cbSymIdTypeVal.EnableWindow( bActive );
		m_chkTimestamp.EnableWindow( bActive );

		// If we don't have an active connection, then we are done initializing
		if (!bActive)
			return 0;							// Allow activate

		HRESULT hr = S_OK;
		const HRESULT HR_WAIT_TIMEOUT = HRESULT_FROM_WIN32( WAIT_TIMEOUT );

		// Fetch the boolean symbology property value and set the control
		CWaitCursor wait;
		int nSymPropSel = m_cbSymProp.GetCurSel();
		if (nSymPropSel >= 0) {
			TCHAR szSymProp[32];
			int nChars = m_cbSymProp.GetLBText( nSymPropSel, szSymProp );
			ATLASSERT( nChars < sizeof(szSymProp)/sizeof(TCHAR) );
			CComVariant varSymProp;
			hr = g_pConnection->GetProp( szSymProp, &varSymProp );
			if (SUCCEEDED(hr) && varSymProp.vt == VT_BOOL)
				m_cbSymPropVal.SetCurSel( varSymProp.boolVal ? 0 : 1 );
		}

		// Fetch the boolean symbology identifier type value and set the control
		if (hr != HR_WAIT_TIMEOUT) {
			CComVariant varSymIdType;
			hr = g_pConnection->GetProp( _T("SymIdType"), &varSymIdType );
			if (SUCCEEDED(hr) && varSymIdType.vt == VT_BSTR )
				::SendMessageW( m_cbSymIdTypeVal, CB_SELECTSTRING, -1, (LPARAM)(PCWSTR)varSymIdType.bstrVal );
		}

		// Fetch the boolean timestamp property value and set the control
		if (hr != HR_WAIT_TIMEOUT) {
			CComVariant varTimestamp;
			hr = g_pConnection->GetProp( _T("Timestamp"), &varTimestamp );
			if (SUCCEEDED(hr) && varTimestamp.vt == VT_BOOL )
				m_chkTimestamp.SetCheck( varTimestamp.boolVal );
		}

		return 0;							// Allow activate
	}

	// Constructors
//	CSettingsPage() {}

	// Overrides

	// Attributes
};

class COptionsPage : public CPropertyPageImpl<COptionsPage>
{
	// Dialog controls
	CButton		m_chkAutoActivate;	// Automatically active the scanner on application start up
	CButton		m_chkAutoRun;		// Automatically run the application at Windows start up
	CButton		m_chkStartHidden;	// Do not show the dialog on application start up
 
	// Member variables
	bool		m_bAutoActivate;
	bool		m_bAutoRun;
	bool		m_bStartHidden;
	TCHAR		m_szModule[_MAX_PATH];		// Module file name

    // Support functions
	LONG SaveSettings()						// Save Option settings
	{
		if (!IsWindow())					// Nothing to do!
			return ERROR_SUCCESS;

		// Open the Run registry key
		CRegKey rkRun;
		LONG lRes = rkRun.Open( HKEY_CURRENT_USER, pcszRegKeyRun );
		ATLASSERT( lRes == ERROR_SUCCESS );
		if (lRes != ERROR_SUCCESS)
			return lRes;

		// Add or remove the application name entry
		m_bAutoRun = m_chkAutoRun.GetCheck() != FALSE;
		lRes = m_bAutoRun ? rkRun.SetStringValue(pcszAppName, m_szModule) : rkRun.DeleteValue(pcszAppName);

		// Close the registry key
		rkRun.Close();

		// Open the App registry key
		CRegKey rkApp;
		lRes = rkApp.Create( regHive.GetWritableHive(), pcszAppRegKey );
		ATLASSERT( lRes == ERROR_SUCCESS );
		if (lRes != ERROR_SUCCESS)
			return lRes;

		// Set m_bAutoActivate value;
		DWORD dwValue = m_chkAutoActivate.GetCheck();
		lRes = rkApp.SetDWORDValue( _T("AutoActivate"), dwValue);
		m_bAutoActivate = dwValue != FALSE;

		// Set m_bStartHidden value;
		dwValue = m_chkStartHidden.GetCheck();
		lRes = rkApp.SetDWORDValue( _T("StartHidden"), dwValue);
		m_bStartHidden = dwValue != FALSE;

		if (lRes == ERROR_SUCCESS)
			SetModified( false );			// This will disable the Apply button

		return lRes;
	}

	LONG LoadSettings()						// Load Option settings
	{
		// Open the App registry key
		CRegKey rkApp;
		LONG lRes = rkApp.Open( regHive, pcszAppRegKey, KEY_READ );
		if (lRes != ERROR_SUCCESS)
			return lRes;

		// Set m_bAutoActivate value;
		DWORD dwValue = FALSE;
		lRes = rkApp.QueryDWORDValue( _T("StartHidden"), dwValue);
		//REG ATLASSERT( lRes == ERROR_SUCCESS );
		m_bStartHidden = dwValue != FALSE;
		if (m_chkStartHidden)
			m_chkStartHidden.SetCheck( m_bStartHidden );

		// Set m_bStartHidden value;
		dwValue = FALSE;
		lRes = rkApp.QueryDWORDValue( _T("AutoActivate"), dwValue);
		//REG ATLASSERT( lRes == ERROR_SUCCESS );
		m_bAutoActivate = dwValue != FALSE;
		if (m_chkAutoActivate)
			m_chkAutoActivate.SetCheck( m_bAutoActivate );

		// Close the registry key
		lRes = rkApp.Close();
		ATLASSERT( lRes == ERROR_SUCCESS );
		if (lRes != ERROR_SUCCESS)
			return lRes;

		// Open the Run registry key
		CRegKey rkRun;
		lRes = rkRun.Open( HKEY_CURRENT_USER, pcszRegKeyRun, KEY_READ );
		ATLASSERT( lRes == ERROR_SUCCESS );
		if (lRes != ERROR_SUCCESS)
			return lRes;

		// Check for application name entry
		TCHAR szValue[_MAX_PATH];
		ULONG nChars = _MAX_PATH;
		lRes = rkRun.QueryStringValue( pcszAppName, szValue, &nChars);
		m_bAutoRun = lRes == ERROR_SUCCESS && _tcsicmp(szValue, m_szModule) == 0;
		if (m_chkAutoRun)
			m_chkAutoRun.SetCheck( m_bAutoRun );

		if (m_hWnd)
			SetModified( false );			// This will disable the Apply button

		return ERROR_SUCCESS;
	}

    // Maps
    BEGIN_MSG_MAP_EX(COptionsPage)
		MSG_WM_INITDIALOG( OnInitDialog )
		COMMAND_CODE_HANDLER_EX( BN_CLICKED,	OnClicked )
        CHAIN_MSG_MAP(CPropertyPageImpl<COptionsPage>)
    END_MSG_MAP()

    // Message handlers
	BOOL OnInitDialog( HWND /*hWndFocus*/, LPARAM /*lParam*/ )
	{
		// Initialize button objects
		m_chkAutoRun		= GetDlgItem( IDC_AUTO_RUN );		// Automatically run the application at Windows start up
		m_chkStartHidden	= GetDlgItem( IDC_START_HIDDEN );	// Hide the dialog on application start up
		m_chkAutoActivate	= GetDlgItem( IDC_AUTO_ACTIVATE );	// Automatically active the scanner on application start up

		// Initialize dialog to match member variables
		m_chkStartHidden.SetCheck(	m_bStartHidden );
		m_chkAutoActivate.SetCheck(	m_bAutoActivate );
		m_chkAutoRun.SetCheck(		m_bAutoRun );

		return TRUE;
	}

	void OnClicked( UINT uNotifyCode, int /*nID*/, HWND /*hWndCtrl*/ )
	{
		SetModified();
		SetMsgHandled( false );
	}

public:
    enum { IDD = IDD_OPTIONS };

	// Constructors
	COptionsPage() {
		::GetModuleFileName(_pModule->GetModuleInstance(), m_szModule, _MAX_PATH);
		LoadSettings();	// Ignore errors
	}

	// Overrides
	int OnApply()		{ return SaveSettings() == ERROR_SUCCESS ? PSNRET_NOERROR : PSNRET_INVALID; }
	void OnReset()		{ LoadSettings(); }

	// Attributes
	bool AutoActivate()	{ return m_bAutoActivate; }
	bool StartHidden()	{ return m_bStartHidden; }
};
